#include <ygNodeDB.h>
#include <ygUtil.h>
#include "MiletusNavigator.h"

using namespace std;

extern "C" ygNode* construct_MiletusNavigator(const char* name,bool master) { return new MiletusNavigator(name,master); }

struct _MiletusNavigatorPrivateData
	{
	ygNode * attachNode;
	pfMatrix attachNodePrevMatrix;
	};


MiletusNavigator::MiletusNavigator(const char* name,bool master) : ygCAVENavigator(name,master)
	{
	setClassName("MiletusNavigator");
	p_ = new struct _MiletusNavigatorPrivateData;
	p_->attachNode = NULL;
	}

MiletusNavigator::~MiletusNavigator(void)
	{
	delete p_;
	}


void MiletusNavigator::reset(void)
	{
	p_->attachNode = NULL;
	ygCAVENavigator::reset();
	}


void MiletusNavigator::message(const ygMessage& msg)
	{
	if (msg == "attach")
		attach(msg.args[0]);
	else if (msg == "release")
		release();
	else
		ygCAVENavigator::message(msg);
	}


void MiletusNavigator::attach(const ygString& nodeName)
	{
	if (p_->attachNode)
		release();
	p_->attachNode = ygNodeDB::find(nodeName);
	if (p_->attachNode)
		p_->attachNode->getTransform(p_->attachNodePrevMatrix);
	setFly(true);
	setCollide(false);
	}


void MiletusNavigator::release(void)
	{
	p_->attachNode = NULL;
	setFly(false);
	setCollide(true);
	}


void MiletusNavigator::updateAttach(void)
	{
	pfMatrix prevInverse, newMatrix, delta;
	pfVec3 newPos, dir, newDir;
	prevInverse.invertAff(p_->attachNodePrevMatrix);
	p_->attachNode->getTransform(newMatrix);
	delta = prevInverse * newMatrix;
	newPos.xformPt(data().position(),delta);
	data().setPosition(newPos);
	pfSinCos(data().heading(), &dir[0], &dir[1]);
	dir[0] = -dir[0];
	dir[2] = 0;
	newDir.xformVec(dir,delta);
	data().setHeading(pfArcTan2(-newDir[0],newDir[1]));
	p_->attachNodePrevMatrix = newMatrix;
	}


void MiletusNavigator::app(void)
	{
	if (p_->attachNode)
		updateAttach();
	ygCAVENavigator::app();
	}
