#include <Performer/pf/pfGroup.h>
#include <set>
#include "ygWorld.h"
#include "ygUser.h"
#include "ygWand.h"
#include "PointAtTrigger.h"

using namespace std;

extern "C" ygNode* construct_PointAtTrigger(const char* name,bool master) { return new PointAtTrigger(name,master); }

struct _PointAtTriggerPrivateData
	{
	set<ygWand*> prevPointing;
	float distance;
	};


PointAtTrigger::PointAtTrigger(const char* name,bool master) : ygNode(name,master)
	{
	setClassName("PointAtTrigger");
	p_ = new struct _PointAtTriggerPrivateData;
	p_->distance = 10;
	}

PointAtTrigger::~PointAtTrigger(void)
	{
	delete p_;
	}


void PointAtTrigger::reset(void)
	{
	p_->prevPointing.clear();
	p_->distance = 10;
	ygNode::reset();
	}


void PointAtTrigger::message(const ygMessage& msg)
	{
	if (msg == "distance")
		p_->distance = msg.floatArg(0);
	else
		ygNode::message(msg);
	}


void PointAtTrigger::app(void)
	{
	set<ygWand*> pointing;
	int i,j,f;
	pfMatrix myXform, myXformInv;
	getTransform(myXform);
	myXformInv.invertAff(myXform);
	for (i=0; i < ygWorld::World->numUsers(); i++)
		{
		ygUser * user = ygWorld::World->user(i);
		for (j=0; j < user->numWands(); j++)
		    {
		    if ((user->wand(j)) && (user->wand(j)->isOfClass("ygWand")))
			{
			ygWand * wand = (ygWand*) user->wand(j);
			if (pointingAtMe(wand,myXformInv))
				{
				pointing.insert(pointing.begin(),wand);
				if (p_->prevPointing.find(wand) == p_->prevPointing.end())
					{
					ygString args("wand=");
					args += wand->name();
					args += " user=";
					args += user->name();
					eventOccurred("start",args);
					}
				if (wand->numFlags() > 0)
					{
					ygString event, args("wand=");
					args += wand->name();
					args += " user=";
					args += user->name();
					for (f=0; f < wand->numFlags(); f++)
						{
						wand->getFlag(f,event);
						eventOccurred(event,args);
						}
					}
				}
			}
		    }
		}
	set<ygWand*>::const_iterator iter;
	for (iter = p_->prevPointing.begin(); iter != p_->prevPointing.end();
	     ++iter)
		if (pointing.find(*iter) == pointing.end())
			{
			ygString args("wand=");
			args += (*iter)->name();
			args += " user=";
			args += (*iter)->user()->name();
			eventOccurred("stop",args);
			}
	p_->prevPointing = pointing;
	ygNode::app();
	}


bool PointAtTrigger::pointingAtMe(const ygWand* wand,const pfMatrix& myXformInv) const
	{
	pfSegSet segset;
	pfHit **hits[32];
	pfMatrix wandXform;
	pfVec3 forward(0, 1, 0), wandDir;
	segset.activeMask = 1;
	segset.isectMask = 0xFFFF;
	segset.discFunc = NULL;
	segset.bound = NULL;
	segset.mode = PFTRAV_IS_PRIM;
	segset.segs[0].pos = wand->origin(this);
	wand->getTransform(wandXform);
	wandDir.xformVec(forward, wandXform);
	wandDir.xformVec(wandDir, myXformInv);
	segset.segs[0].dir = wandDir;
	segset.segs[0].length = p_->distance;
	return pfnode()->isect(&segset, hits);
	}
