#include <ygUser.h>
#include <ygWorld.h>
#include <ygUtil.h>
#include <set>
#include "UserTrigger.h"

using namespace std;


extern "C" ygNode* construct_UserTrigger(const char* name,bool master) { return new UserTrigger(name,master); }


struct _UserTriggerPrivateData
	{
	set<ygUser*> prevInside;
	bool debugInside;
	int lastFrame;
	};


UserTrigger::UserTrigger(const char* name,bool master) : ygSpace(name,master)
	{
	setClassName("UserTrigger");
	p_ = new struct _UserTriggerPrivateData;
	p_->lastFrame = ygWorld::FrameNumber;
	debugFlag("inside", &p_->debugInside);
	}


UserTrigger::~UserTrigger(void)
	{
	delete p_;
	}


void UserTrigger::reset(void)
	{
	p_->prevInside.clear();
	p_->lastFrame = ygWorld::FrameNumber;
	ygSpace::reset();
	}


void UserTrigger::app(void)
	{
	set<ygUser*> inside;
	int i;
	if (p_->lastFrame < (ygWorld::FrameNumber-1))
		p_->prevInside.clear();
	p_->lastFrame = ygWorld::FrameNumber;
	for (i=0; i < ygWorld::World->numUsers(); i++)
		{
		ygUser * user = ygWorld::World->user(i);
		if ((user->head()) &&
		    (contains(user->head()->origin(this))))
			{
			if (p_->debugInside)
				cout << name() << ": " << user->name() << " is inside\n";
			inside.insert(inside.begin(),user);
			if (p_->prevInside.find(user) == p_->prevInside.end())
				{
				ygString args("user=");
				args += user->name();
				eventOccurred("enter",args);
				}
			}
		}
	set<ygUser*>::const_iterator iter;
	for (iter = p_->prevInside.begin(); iter != p_->prevInside.end();
	     ++iter)
		if (inside.find(*iter) == inside.end())
			{
			ygString args("user=");
			args += (*iter)->name();
			eventOccurred("exit",args);
			}
	if ((p_->prevInside.size() > 0) && (inside.size() == 0))
		eventOccurred("empty");
	if ((p_->prevInside.size() == 0) && (inside.size() > 0))
		{
		ygString args("user=");
		args += (*(inside.begin()))->name();
		eventOccurred("firstenter",args);
		}
	p_->prevInside = inside;
	ygSpace::app();
	}
