#include <Performer/pf/pfSwitch.h>
#include <vector>
#include "ygNodeDB.h"
#include "ygSpace.h"
#include "ygVolume.h"
#include "ygWorld.h"
#include "ygNetKeys.h"
#include "Visibility.h"

using namespace std;

extern "C" ygNode* construct_Visibility(const char* name,bool master) { return new Visibility(name,master); }

enum {
	ALWAYS_ON,
	ALWAYS_OFF,
	NORMAL
	};

struct _VisibilityPrivateData
	{
	pfSwitch * pfswitch;
	vector<ygSpace*> space;
	bool inside;
	int mode;
	ygString netString;
	};


Visibility::Visibility(const char* name,bool master) : ygNode(name,master,false)
	{
	setClassName("Visibility",true);
	p_ = new struct _VisibilityPrivateData;
	p_->inside = true;
	p_->mode = NORMAL;
	p_->pfswitch = new pfSwitch;
	setPfNode(p_->pfswitch);
	addNetKey("space", &p_->netString, YG_NET_STRING);
	addNetKey("in", &p_->inside, YG_NET_BOOL);
	addNetKey("mode", &p_->mode, YG_NET_INT);
	}

Visibility::~Visibility(void)
	{
	delete p_;
	}


void Visibility::reset(void)
	{
	p_->space.clear();
	setInside(true);
	if (isNetMaster())
		{
		p_->netString.clear();
		netKeyChanged("space");
		}
	ygNode::reset();
	}


void Visibility::message(const ygMessage& msg)
	{
	if (msg == "inside")
		setInside(true);
	else if (msg == "outside")
		setInside(false);
	else if (msg == "space")
		addSpace(msg.args[0]);
	else if (msg == "mode")
		setMode(msg.args[0]);
	else
		ygNode::message(msg);
	}


void Visibility::setInside(bool val)
	{
	p_->inside = val;
	netKeyChanged("in");
	}


void Visibility::setMode(const ygString& modestr)
	{
	if (modestr == "on")
		p_->mode = ALWAYS_ON;
	else if (modestr == "off")
		p_->mode = ALWAYS_OFF;
	else
		p_->mode = NORMAL;
	netKeyChanged("mode");
	}


void Visibility::addSpace(const ygString& name)
	{
	ygNode * spaceNode = ygNodeDB::find(name);
	if ((spaceNode) && (spaceNode->isOfClass("ygSpace")))
		{
		p_->space.push_back((ygSpace*)spaceNode);
		if (isNetMaster())
			{
			p_->netString += " ";
			p_->netString += name;
			netKeyChanged("space");
			}
		}
	}


void Visibility::app(void)
	{
	ygNode * viewer = ygWorld::World->viewer();
	if (p_->mode == ALWAYS_OFF)
		p_->pfswitch->setVal(PFSWITCH_OFF);
	else if ((!viewer) || (p_->space.size() == 0) || (p_->mode == ALWAYS_ON))
		p_->pfswitch->setVal(PFSWITCH_ON);
	else
		{
		bool viewerInside = false;
		int i;
		for (i=0; i < p_->space.size(); i++)
			{
			if (p_->space[i]->contains(viewer->origin(p_->space[i])))
				{
				viewerInside = true;
				break;
				}
			}
		if ((viewerInside && p_->inside) || ((!viewerInside) && (!p_->inside)))
			p_->pfswitch->setVal(PFSWITCH_ON);
		else
			p_->pfswitch->setVal(PFSWITCH_OFF);
		}
	ygNode::app();
	}


void Visibility::acceptNetKey(const ygString& key)
	{
	if (key == "space")
		{
		ygString * spacename;
		int i=0;
		p_->space.clear();
		while (spacename = p_->netString.nextToken(" ",&i))
			{
			addSpace(*spacename);
			delete spacename;
			}
		}
	else if (key == "in")
		;
	else
		ygNode::acceptNetKey(key);
	}
