#include <ygUser.h>
#include <ygWand.h>
#include <ygUtil.h>
#include <ygWorld.h>
#include <vector>
#include "WandTrigger.h"

using namespace std;

extern "C" ygNode* construct_WandTrigger(const char* name,bool master) { return new WandTrigger(name,master); }

struct _WandTriggerPrivateData
	{
	vector<ygWand*> prevInside;
	vector<ygUser*> prevInsideUser;
	int lastFrame;
	};


WandTrigger::WandTrigger(const char* name,bool master) : ygSpace(name,master)
	{
	setClassName("WandTrigger");
	p_ = new struct _WandTriggerPrivateData;
	p_->lastFrame = 0;
	}

WandTrigger::~WandTrigger(void)
	{
	delete p_;
	}


void WandTrigger::reset(void)
	{
	p_->prevInside.clear();
	p_->prevInsideUser.clear();
	p_->lastFrame = ygWorld::FrameNumber;
	ygSpace::reset();
	}


void WandTrigger::app(void)
	{
	vector<ygWand*> inside;
	vector<ygUser*> insideUser;
	int i,j,f;
	if (p_->lastFrame < (ygWorld::FrameNumber-1))
		{
		p_->prevInside.clear();
		p_->prevInsideUser.clear();
		}
	p_->lastFrame = ygWorld::FrameNumber;
	for (i=0; i < ygWorld::World->numUsers(); i++)
		{
		ygUser * user = ygWorld::World->user(i);
		for (j=0; j < user->numWands(); j++)
		    {
		    if ((user->wand(j)) && (user->wand(j)->isOfClass("ygWand")))
			{
			ygWand * wand = (ygWand*) user->wand(j);
			if (contains(wand->origin(this)))
				{
				inside.push_back(wand);
				insideUser.push_back(user);
				if (!previouslyInside(wand))
					{
					ygString args("wand=");
					args += wand->name();
					args += " user=";
					args += user->name();
					eventOccurred("enter",args);
					}
				if (wand->numFlags() > 0)
					{
					ygString event, args("wand=");
					args += wand->name();
					args += " user=";
					args += user->name();
					for (f=0; f < wand->numFlags(); f++)
						{
						wand->getFlag(f,event);
						eventOccurred(event,args);
						}
					}
				}
			}
		    }
		}
	p_->prevInside.swap(inside);
	p_->prevInsideUser.swap(insideUser);
	for (i=0; i < inside.size(); i++)
		if (!previouslyInside(inside[i]))
			{
			ygString args("wand=");
			args += inside[i]->name();
			if (insideUser[i])
				{
				args += " user=";
				args += insideUser[i]->name();
				}
			eventOccurred("exit",args);
			}
	ygSpace::app();
	}


bool WandTrigger::previouslyInside(const ygWand *w) const
	{
	int i;
	for (i=0; i < p_->prevInside.size(); i++)
		if (w == p_->prevInside[i])
			return true;
	return false;
	}
