#include "ygNodeDB.h"
#include "audioTool.h"

extern "C" ygNode* construct_audioTool(const char* name,bool master) { return new audioTool(name,master); }
using namespace std;


audioTool::audioTool(const char* name,bool master) : ygNode(name,master)
	{
	setClassName("audioTool",true);
	playing_ = false;
	}


void audioTool::reset(void)
	{
	playing_ = false;
	oldQueue_.clear();
	newQueue_.clear();
	playedSounds_.clear();
	ygNode::reset();
	}

// e.g. queue(nameofSoundNode)
void audioTool::message(const ygMessage& msg)
	{
	if (msg == "language")
		language_ = msg.args[0];
		
	else if (msg == "queue")
	{
		queueSound(msg.args[0]);
	}

	else if (msg == "play")
	{
		playSound(msg.args[0]);
	}
	
	else if (msg == "toggle")
	{
		toggleSound();
	}
	
	else if (msg == "stopped")
	{
		playing_ = false;
		checkQueues();
	}
	
	else if (msg == "deQueue")
	{
		clearQueue();
	}
	
	
	else
		ygNode::message(msg);
	}




void audioTool::queueSound(ygString nameofSoundNode)
{ 
	//check to see if its been played
	//if so put in oldQueue, if not put in newQueue
	if (alreadyPlayed(nameofSoundNode))
	{
	   oldQueue_.push_back(nameofSoundNode);
	}
		   
	else
	{
	    newQueue_.push_back(nameofSoundNode);
	}

	checkQueues();
}


bool audioTool::alreadyPlayed(ygString nameofSoundNode)
{
    for(int i = 0; i < playedSounds_.size() ; i++)
    {
	if(playedSounds_[i] == nameofSoundNode)
	   {
	       return true; 
	   }
     }
    
    return false; 
}

void audioTool::toggleSound(void)
{ 
    if (!playing_)
	selectSound();
    else
    	stopSound();
}

void audioTool::selectSound(void)
{
    //how do we choose the sound? to start with make it the top of newQueue
    ygString nameofSoundNode;
    
    if(!newQueue_.empty())
    {
	nameofSoundNode = newQueue_[0];
	playSound(nameofSoundNode);
	    //add sound to playedSound Queue and oldQueue
	playedSounds_.push_back(nameofSoundNode);
	oldQueue_.push_back(nameofSoundNode);
	    //pop sound off newQueue
	newQueue_.pop_front();
    }
    
    else if(!oldQueue_.empty())
    {
	nameofSoundNode = oldQueue_[0];
	playSound(nameofSoundNode);
	    //add sound on the back
	oldQueue_.push_back(nameofSoundNode);
	    //pop sound off front
	oldQueue_.pop_front();
   }
}

void audioTool::playSound(ygString nameofSoundNode)
{
	//find sound node
    ygNode *node = ygNodeDB::find(nameofSoundNode);
    if ((node) && (node->isOfClass("soundNode")))
	{
	soundNode * theSound = (soundNode*) node;
	ygString soundfile;
	if (theSound->soundFile(language_, soundfile))
	    {
		//cout << "soundfile for " << language_ << " is " << soundfile << endl;
		//eventoccured out to the ygSound with soundfile
		//first build up the eventoccurred message
	    ygString args("file=");
	    args += soundfile;
	    eventOccurred("play",args);
	    }
	playing_ = true;  
	}
	
    else
	cout<<"Could not find soundNode named " << nameofSoundNode
	    <<endl;       
}

void audioTool::stopSound(void)
{
    eventOccurred("stop");
    playing_ = false;  
}


void audioTool::checkQueues(void)
{	
    if (!newQueue_.empty())
	eventOccurred("queued");
    else if (!oldQueue_.empty())
    {
	eventOccurred("playedSoundQueued");
	cout<<"playedSoundQueued"<<endl;
    }
    else
	eventOccurred("queueCleared");
}


void audioTool::clearQueue(void)
{
    oldQueue_.clear();
    newQueue_.clear();
    eventOccurred("queueCleared");
}
