#include <ygWorld.h>
#include "bouncer.h"

using namespace std;

extern "C" ygNode* construct_bouncer(const char* name,bool master) { return new bouncer(name,master); }

struct _bouncerPrivateData
	{
	float height, duration;
	bool active;
	float startTime;
	bool debugHeight;
	};


bouncer::bouncer(const char* name,bool master) : ygTransform(name,master)
	{
	setClassName("bouncer");
	p_ = new struct _bouncerPrivateData;
	p_->height = 1.0;
	p_->duration = 1.0;
	p_->active = false;
	debugFlag("height", &p_->debugHeight);
	}

bouncer::~bouncer(void)
	{
	delete p_;
	}


void bouncer::reset(void)
	{
	ygTransform::reset();
	}


void bouncer::message(const ygMessage& msg)
	{
	if (msg == "height")
		{
		setHeight(msg.floatArg(0));
		}
	else if (msg == "duration")
		{
		setDuration(msg.floatArg(0));
		}
	else if (msg == "bounce")
		{
		bounce();
		}
	else
		ygTransform::message(msg);
	}


void bouncer::setHeight(float height)
	{
	p_->height = height;
	}


void bouncer::setDuration(float duration)
	{
	p_->duration = duration;
	}


void bouncer::bounce(void)
	{
	p_->active = true;
	p_->startTime = ygWorld::FrameTime;
	}


void bouncer::app(void)
	{
	if (p_->active)
		{
		float t = ygWorld::FrameTime - p_->startTime;
		if (t < p_->duration)
			{
			float z = sin(t / p_->duration * M_PI) * p_->height;
			setPosition(0.0, 0.0, z);
			if (p_->debugHeight)
				cout << name() << ": height is " << z << endl;
			}
		else
			{
			setPosition(0.0, 0.0, 0.0);
			p_->active = false;
			char str[32];
			sprintf(str,"height=%f duration=%f",p_->height,p_->duration);
			eventOccurred("finish", str);
			}
		}
	ygTransform::app();
	}
