#include <stdio.h>
#include <math.h>
#include <string.h>
#include <malloc.h>
#include <Performer/pr/pfGeoSet.h>
#include <Performer/pr/pfGeoState.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pf/pfDCS.h>
#include <Performer/pfdu.h>
#include <vector>
#include "ygWorld.h"
#include "ygUser.h"
#include "mapper.h"

using namespace std;

extern "C" ygNode* construct_mapper(const char* name,bool master) { return new mapper(name,master); }

struct _mapperPrivateData
	{
	pfGeode * mySphere,  * otherSphere;
	pfGroup * sphereParent;
	vector<pfDCS*> sphereDCS;
	};

static pfGeode * createSphere(float radius,pfVec4 color,int numTris)
{
 pfGeoSet *gset;
 pfVec4 *colorAttr;
 pfMatrix matrix;
 pfGeode *geode = new pfGeode;
 gset = pfdNewSphere(numTris,pfGetSharedArena());
 gset->setGState(new pfGeoState);
 colorAttr = (pfVec4 *) pfMalloc(sizeof(pfVec4),pfGetSharedArena());
 *colorAttr = color;
 gset->setAttr(PFGS_COLOR4,PFGS_OVERALL,colorAttr,NULL);
 matrix.makeScale(radius,radius,radius);
 pfdXformGSet(gset,matrix);
 geode->addGSet(gset);
 geode->setTravMask(PFTRAV_ISECT, 0, PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
 return geode;

}

mapper::mapper(const char* name,bool master) : ygNode(name,master)
	{
	setClassName("mapper");
	p_ = new struct _mapperPrivateData;
	pfVec4 color(1, 1, 0, 1);
	p_->mySphere = createSphere(0.03, color, 24);
	color.set(0, 0, 1, 1);
	p_->otherSphere = createSphere(0.03, color, 24);
	p_->sphereParent = new pfGroup;
	pfnode()->addChild(p_->sphereParent);
	}

mapper::~mapper(void)
	{
	delete p_;
	}


void mapper::reset(void)
	{
	ygNode::reset();
	}


void mapper::message(const ygMessage& msg)
	{
	if (msg == "userrange")
	{
		msg.getVec2Args(userMin_);
		msg.getVec2Args(userMax_,  2);
	}
	else if (msg == "maprange")
	{
		msg.getVec3Args(map_.min);
		msg.getVec3Args(map_.max,  3);
	}
	else if (msg == "printstate")
	{
		printState();
	}
	else
		ygNode::message(msg);
	}


void mapper::app(void)
	{
	pfGroup * parent = p_->sphereParent;
	pfVec3 userPos;
	int i;
	while (parent->getNumChildren())
	    parent->removeChild(parent->getChild(0));
	for (i=0; i < ygWorld::World->numUsers(); i++)
		{
		ygUser * user = ygWorld::World->user(i);
		pfVec3 userPos = user->origin();
		float x, y, mapX, mapY, mapZ;
		x = (userPos[0] - userMin_[0]) / (userMax_[0] - userMin_[0]);
		y = (userPos[1] - userMin_[1]) / (userMax_[1] - userMin_[1]);
		if (x < 0)
			x = 0;
		else if (x > 1)
			x = 1;
		if (y < 0)
			y = 0;
		else if (y > 1)
			y = 1;
		mapX = map_.min[0] + x * (map_.max[0] - map_.min[0]);
		mapY = map_.min[1] + y * (map_.max[1] - map_.min[1]);
		mapZ = map_.min[2] + y * (map_.max[2] - map_.min[2]); 
		if (i >= p_->sphereDCS.size())
		    {
			pfDCS * dcs = new pfDCS;
			if (i > 0)
			    dcs->addChild(p_->otherSphere);
			else
			    dcs->addChild(p_->mySphere);
			p_->sphereDCS.push_back(dcs);
		    }
		parent->addChild(p_->sphereDCS[i]);
		p_->sphereDCS[i]->setTrans(mapX, mapY, mapZ);
		}

	ygNode::app();
	}


void mapper::printState(void)
	{
	pfVec3 userPos;
	int i;
	for (i=0; i < ygWorld::World->numUsers(); i++)
		{
		ygUser * user = ygWorld::World->user(i);
		pfVec3 userPos = user->origin();
		float x, y;
		x = (userPos[0] - userMin_[0]) / (userMax_[0] - userMin_[0]);
		y = (userPos[1] - userMin_[1]) / (userMax_[1] - userMin_[1]);
		if (x < 0)
			x = 0;
		else if (x > 1)
			x = 1;
		if (y < 0)
			y = 0;
		else if (y > 1)
			y = 1;
		printf("%s: %s is at (%f,%f,%f) = map pos (%f,%f)\n",name().c_str(),
			user->name().c_str(), userPos[0],userPos[1],userPos[2],x,y);
		}
	}
