#include <malloc.h>
#include <Performer/pf/pfSwitch.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pr/pfGeoSet.h>
#include <Performer/pr/pfTexture.h>
#include <dmedia/moviefile.h>
#include "ygUtil.h"
#include "ygWorld.h"
#include "ygNetKeys.h"
#include "movieTexture.h"

using namespace std;

extern "C" ygNode* construct_movieTexture(const char* name,bool master) { return new movieTexture(name,master); }

struct _movieTexturePrivateData
	{
	pfTexture * texture;
	bool active;
	float startTime;
	MVid movie, track;
	int width,height,buffersize;
	float rate;
	void *buffer;
	ygString netFile;
	bool firstFrame;
	bool debugFrame;
	};


movieTexture::movieTexture(const char* name,bool master) : ygNode(name,master)
	{
	setClassName("movieTexture",true);
	p_ = new struct _movieTexturePrivateData;
	p_->firstFrame = true;
	p_->active = false;
	p_->movie = NULL;
	addNetKey("file", &p_->netFile, YG_NET_STRING);
	debugFlag("frame", &p_->debugFrame);
	}

movieTexture::~movieTexture(void)
	{
	delete p_;
	}


void movieTexture::reset(void)
	{
	ygNode::reset();
	}


void movieTexture::message(const ygMessage& msg)
	{
	if (msg == "movie")
		{
		if (msg.args.size() == 1)
			setMovie(msg.args[0]);
		else
			msg.error(name(), "(wrong number of arguments)");
		}
	else if (msg == "play")
		play();
	else
		ygNode::message(msg);
	}


void movieTexture::setMovie(const ygString& file)
	{
	char *searchPath = ".";
	ygString fullPath;
	if (getenv("YG_PATH"))
		searchPath = getenv("YG_PATH");
	if (!ygFindFile(file,searchPath,fullPath))
		{
		fprintf(stderr,"ERROR: movieTexture::show: could not find file '%s'\n",
			file.c_str());
		return;
		}
	if (p_->movie)
		mvClose(p_->movie);
	if (mvOpenFile(fullPath.c_str(),O_RDONLY,&p_->movie) == DM_FAILURE)
		{
		fprintf(stderr,"ERROR (movieTexture::show): failed to open '%s'\n",
			fullPath.c_str());
		fprintf(stderr,"%s\n",mvGetErrorStr(mvGetErrno()));
		mvClearErrno();
		return;
		}
	if (mvFindTrackByMedium(p_->movie,DM_IMAGE,&p_->track) == DM_FAILURE)
		{
		fprintf(stderr,"ERROR (movieTexture::show): failed to find image track in '%s'\n",fullPath.c_str());
		fprintf(stderr,"%s\n",mvGetErrorStr(mvGetErrno()));
		mvClearErrno();
		mvClose(p_->movie);
		return;
		}
	p_->width = mvGetImageWidth(p_->track);
	p_->height = mvGetImageHeight(p_->track);
	p_->rate = mvGetImageRate(p_->track);
	p_->buffersize = p_->width * p_->height * sizeof(long);
	p_->buffer = pfMalloc(p_->buffersize, pfGetSharedArena());
	p_->netFile = file;
	netKeyChanged("file");
	}


void movieTexture::play(void)
	{
		p_->active = true;
		p_->startTime = ygWorld::FrameTime;
	}

void movieTexture::app(void)
	{
	if (p_->movie)
		{
		if (p_->firstFrame)
			{
				createTexture();
				replaceTextures(pfnode());
				p_->firstFrame = false;
			}
		if (p_->active)
			{
			int frame = (ygWorld::FrameTime - p_->startTime) * p_->rate;
			if (p_->debugFrame)
				cout << "Reading frame " << frame << " of " << p_->netFile << endl;
			if (mvReadFrames(p_->track,frame,1,p_->buffersize,p_->buffer) ==
					DM_SUCCESS)
				{
				p_->texture->setFormat(PFTEX_IMAGE_FORMAT, GL_ABGR_EXT);
				p_->texture->setImage((uint*)p_->buffer,4,p_->width,p_->height,1);
				}
			else
				{
				mvClose(p_->movie);
				p_->active = false;
				eventOccurred("stop");
				}
			}
		}
	ygNode::app();
	}


void movieTexture::createTexture(void)
	{
		p_->texture = new pfTexture;
		if (mvReadFrames(p_->track,0,1,p_->buffersize,p_->buffer) ==
				DM_SUCCESS)
			{
			p_->texture->setFormat(PFTEX_IMAGE_FORMAT, GL_ABGR_EXT);
			p_->texture->setImage((uint*)p_->buffer,4,p_->width,p_->height,1);
			}
	}


void movieTexture::replaceTextures(pfNode * node)
	{
	 int i;
	 if (node->isOfType(pfGeode::getClassType()))
		{
		pfGeode *geode = (pfGeode *)node;
		for (i=0; i < geode->getNumGSets(); i++)
			{
			pfGeoSet *gset = geode->getGSet(i);
			gset->getGState()->setAttr(PFSTATE_TEXTURE, p_->texture);
			}
		}
	 else if (node->isOfType(pfGroup::getClassType()))
		{
		pfGroup *group = (pfGroup *)node;
		for (i=0; i < group->getNumChildren(); i++)
			replaceTextures(group->getChild(i));
		}
	}

void movieTexture::acceptNetKey(const ygString& key)
	{
	if (key == "file")
		{
		setMovie(p_->netFile);
		}
	else
		ygNode::acceptNetKey(key);
	}
