#include <set>
#include <ygNodeDB.h>
#include <ygUtil.h>
#include <ygWorld.h>
#include "nodeTrigger.h"

using namespace std;

extern "C" ygNode* construct_nodeTrigger(const char* name,bool master) { return new nodeTrigger(name,master); }

struct _nodeTriggerPrivateData
	{
	ygString detectType;
	set<ygNode*> prevInside;
	bool debugInside;
	int lastFrame;
	};


nodeTrigger::nodeTrigger(const char* name,bool master) : ygSpace(name,master)
	{
	setClassName("nodeTrigger");
	p_ = new struct _nodeTriggerPrivateData;
	p_->lastFrame = 0;
	debugFlag("inside", &p_->debugInside);
	}

nodeTrigger::~nodeTrigger(void)
	{
	delete p_;
	}


void nodeTrigger::reset(void)
	{
	p_->prevInside.clear();
	p_->lastFrame = ygWorld::FrameNumber;
	ygSpace::reset();
	}


void nodeTrigger::message(const ygMessage& msg)
	{
	if (msg == "detect")
		{
		if (msg.args.size() > 0)
			setDetectType(msg.args[0]);
		else
			msg.error(name(),"(wrong number of arguments)");
		}
	else
		ygSpace::message(msg);
	}


void nodeTrigger::setDetectType(const ygString& s)
	{
	p_->detectType = s;
	}


void nodeTrigger::app(void)
	{
	set<ygNode*> inside;
	int i;
	if (p_->lastFrame < (ygWorld::FrameNumber-1))
		p_->prevInside.clear();
	p_->lastFrame = ygWorld::FrameNumber;
	const vector<ygNode*>& nodes = ygNodeDB::nodesOfType(p_->detectType);
	for (i=0; i < nodes.size(); i++)
		{
		if (contains(nodes[i]->origin(this)))
			{
			if (p_->debugInside)
				cout << name() << ": " << nodes[i]->name() << " is inside\n";
			inside.insert(inside.begin(),nodes[i]);
			if (p_->prevInside.find(nodes[i]) == p_->prevInside.end())
				{
				ygString args("node=");
				args += nodes[i]->name();
				eventOccurred("enter",args);
				}
			}
		}
	set<ygNode*>::const_iterator iter;
	for (iter = p_->prevInside.begin(); iter != p_->prevInside.end();
	     ++iter)
		if (inside.find(*iter) == inside.end())
			{
			ygString args("node=");
			args += (*iter)->name();
			eventOccurred("exit",args);
			}
	p_->prevInside = inside;
	ygSpace::app();
	}
