#include <Performer/pfutil.h>
#include <Performer/pf/pfChannel.h>
#include <Performer/pf/pfTraverser.h>
#include <pfcave.h>
#include "ygWorld.h"
#include "recorder.h"

extern "C" ygNode* construct_recorder(const char* name,bool master) { return new recorder(name,master); }

static int myglFunc(pfTraverser *, void *data);

struct _recorderPrivateData
	{
	bool active;
	int frameNum;
	ygString basename;
	};


recorder::recorder(const char* name,bool master) : ygNode(name,master)
	{
	setClassName("recorder");
	p_ = new struct _recorderPrivateData;
	p_->active = false;
	p_->frameNum = 0;
	p_->basename = "image";
	pfnode()->setTravFuncs(PFTRAV_DRAW, myglFunc, NULL);
	pfnode()->setTravData(PFTRAV_DRAW, p_);
	pfSphere boundingSphere;
	boundingSphere.center.set(0,0,0);
	boundingSphere.radius = 10000.0;
	pfnode()->setBound(&boundingSphere, PFBOUND_STATIC);
	}

recorder::~recorder(void)
	{
	delete p_;
	}


void recorder::reset(void)
	{
	ygNode::reset();
	}


void recorder::message(const ygMessage& msg)
	{
	if (msg == "start")
		p_->active = true;
	else if (msg == "stop")
		p_->active = false;
	else if (msg == "basename")
		p_->basename = msg.args[0];
	else
		ygNode::message(msg);
	}


void recorder::app(void)
	{
	if (p_->active)
		p_->frameNum++;
	ygNode::app();
	}


static int myglFunc(pfTraverser *trav, void *data)
	{
	struct _recorderPrivateData *p = (struct _recorderPrivateData*) data;
	pfChannel *chan = trav->getChan();
	if ((p) && (p->active) && (chan == pfCAVEMasterChan()))
		{
		char filename[512];
		int ox,oy,sx,sy;
		sprintf(filename,"%s%04d.sgi",p->basename.c_str(),p->frameNum);
		chan->getOrigin(&ox,&oy);
		chan->getSize(&sx,&sy);
		pfuSaveImage(filename,ox,oy,sx,sy,0);
		}
	return PFTRAV_CONT;
	}
