#include <ygFileRequest.h>
#include <ygNetKeys.h>
#include <ygNodeDB.h>
#include <ygSound.h>
#include "remoteSound.h"

using namespace std;

extern "C" ygNode* construct_remoteSound(const char* name,bool master) { return new remoteSound(name,master); }

struct _remoteSoundPrivateData
	{
	ygString player;
	ygString file;
	ygString directory;
	bool play;
	ygFileRequest * upload, * download;
	ygString netFile;
	int netPlay;
	bool debugTransfer;
	};


remoteSound::remoteSound(const char* name,bool master) : ygNode(name,master)
	{
	setClassName("remoteSound");
	p_ = new struct _remoteSoundPrivateData;
	if (getenv("REMOTESOUND_DIRECTORY"))
		p_->directory = getenv("REMOTESOUND_DIRECTORY");
	else
		p_->directory = ".";
	p_->play = false;
	p_->upload = p_->download = 0;
	p_->netPlay = 0;
	addNetKey("player", &p_->player, YG_NET_STRING);
	addNetKey("file", &p_->netFile, YG_NET_STRING);
	addNetKey("play", &p_->netPlay, YG_NET_INT);
	debugFlag("transfer", &p_->debugTransfer);
	}

remoteSound::~remoteSound(void)
	{
	delete p_;
	}


void remoteSound::reset(void)
	{
	ygNode::reset();
	}


void remoteSound::message(const ygMessage& msg)
	{
	if (msg == "player")
		{
		if (msg.args.size() > 0)
			setPlayer(msg.args[0]);
		}
	else if (msg == "send")
		{
		if (msg.args.size() > 0)
			sendFile(msg.args[0]);
		}
	else if (msg == "file")
		{
		if (msg.args.size() > 0)
			setFile(msg.args[0]);
		}
	else if (msg == "play")
		{
		if (msg.args.size() > 0)
			setFile(msg.args[0]);
		play();
		}
	else
		ygNode::message(msg);
	}


void remoteSound::setPlayer(const ygString& file)
	{
	p_->player = file;
	netKeyChanged("player");
	}


void remoteSound::sendFile(const ygString& file)
	{
	ygString remoteName;
	int slashIndex = file.find_last_of("/");
	if (slashIndex == file.npos)
		remoteName = file;
	else
		remoteName.assign(file,slashIndex+1,file.length()-slashIndex-1);
	p_->upload = new ygFileRequest(file, remoteName, ygFileRequest::UPLOAD);
	if (p_->debugTransfer)
		cout << name() << ": uploading '" << file << "' as '" <<
			remoteName << "'\n";
	}


void remoteSound::setFile(const ygString& file)
	{
	fetchFile(file);
	p_->netFile = file;
	netKeyChanged("file");
	}


void remoteSound::fetchFile(const ygString& file)
	{
	ygString localName(p_->directory);
	localName += "/";
	localName += file;
	p_->download = new ygFileRequest(localName, file);
	if (p_->debugTransfer)
		cout << name() << ": downloading '" << file << "' to '" <<
			localName << "'\n";
	}


void remoteSound::play(void)
	{
	p_->netPlay++;
	netKeyChanged("play");
	}


void remoteSound::app(void)
	{
	if ((p_->upload) && (p_->upload->done()))
		uploadFinished();
	if ((p_->download) && (p_->download->done()))
		downloadFinished();
	if ((p_->play) && (p_->file.length() > 0))
		startPlaying();
	ygNode::app();
	}


void remoteSound::uploadFinished(void)
	{
	if (p_->upload->failed())
		cerr << "WARNING: " << name() << ": upload of '" <<
			p_->upload->localFile() << "' failed\n";
	else
		{
		if (p_->debugTransfer)
			cout << name() << ": upload of '" <<
				p_->upload->localFile() << "' completed\n";
		setFile(p_->upload->remoteFile());
		}
	delete p_->upload;
	p_->upload = 0;
	}


void remoteSound::downloadFinished(void)
	{
	if (p_->download->failed())
		cerr << "WARNING: " << name() << ": download of '" <<
			p_->download->remoteFile() << "' failed\n";
	else
		{
		if (p_->debugTransfer)
			cout << name() << ": download of '" << 
				p_->download->remoteFile() << "' completed\n";
		p_->file = p_->download->localFile();
		}
	delete p_->download;
	p_->download = 0;
	}


void remoteSound::startPlaying(void)
	{
	ygSound * playerNode = dynamic_cast<ygSound*> (ygNodeDB::find(
							p_->player, false));
	if (playerNode)
		{
		playerNode->setFile(p_->file);
		playerNode->play();
		}
	else
		cerr << "WARNING: " << name() << ": couldn't find sound node "
			" named '" << p_->player << "'\n";
	p_->play = false;
	}


void remoteSound::acceptNetKey(const ygString& key)
	{
	if (key == "player")
		;
	else if (key == "file")
		fetchFile(p_->netFile);
	else if (key == "play")
		p_->play = true;
	else
		ygNode::acceptNetKey(key);
	}
