#include "ygWorld.h"
#include "simpleMover.h"

using namespace std;

extern "C" ygNode* construct_simpleMover(const char* name,bool master) { return new simpleMover(name,master); }

struct _simpleMoverPrivateData
	{
	pfVec3 start, end;
	float moveTime;
	bool active, forward;
	float startTime;
	};


simpleMover::simpleMover(const char* name,bool master) : ygTransform(name,master)
	{
	setClassName("simpleMover");
	p_ = new struct _simpleMoverPrivateData;
	p_->start.set(0,0,0);
	p_->end.set(0,0,0);
	p_->moveTime = 1;
	p_->active = false;
	p_->forward = true;
	}

simpleMover::~simpleMover(void)
	{
	delete p_;
	}


void simpleMover::reset(void)
	{
	p_->start.set(0,0,0);
	p_->end.set(0,0,0);
	p_->moveTime = 1;
	p_->active = false;
	p_->forward = false;
	ygTransform::reset();
	}


void simpleMover::message(const ygMessage& msg)
	{
	if (msg == "start")
		msg.getVec3Args(p_->start);
	else if (msg == "end")
		msg.getVec3Args(p_->end);
	else if (msg == "time")
		p_->moveTime = msg.floatArg(0);
	else if (msg == "go")
		go();
	else
		ygTransform::message(msg);
	}


void simpleMover::go(void)
	{
	if (!p_->active)
		{
		p_->active = true;
		p_->forward = !p_->forward;
		p_->startTime = ygWorld::FrameTime;
		}
	}


void simpleMover::app(void)
	{
	if (p_->active)
		{
		float t = ygWorld::FrameTime - p_->startTime;
		if (t >= p_->moveTime)
			{
			t = p_->moveTime;
			p_->active = false;
			}
		float a = t / p_->moveTime;
		pfVec3 pos;
		if (p_->forward)
			pos = (1.0f-a) * p_->start + a * p_->end;
		else
			pos = (1.0f-a) * p_->end + a * p_->start;
		setPosition(pos);
		}
	ygTransform::app();
	}
