#include <Performer/pf/pfGroup.h>
#include <ygWorld.h>
#include <ygNetKeys.h>
#include <ygPFObjectCache.h>
#include <Performer/pfdu.h>
#include "staticObject.h"

using namespace std;

extern "C" ygNode* construct_staticObject(const char* name,bool master) { return new staticObject(name,master); }

struct _staticObjectPrivateData
	{
	ygString filename;
	pfNode *obj;
	uint isectMask;
	bool drawn, wall, floor, cache, showLocal;
	ygString netFilename;
	};


staticObject::staticObject(const char* name,bool master) : ygNode(name,master)
	{
	setClassName("staticObject",true);
	p_ = new struct _staticObjectPrivateData;
	p_->obj = NULL;
	p_->isectMask = YG_ISECT_ALLMASK;
	p_->drawn = true;
	p_->cache = true;
	p_->showLocal = true;
	addNetKey("file",&p_->netFilename,YG_NET_STRING);
	addNetKey("draw",&p_->drawn,YG_NET_BOOL);
	addNetKey("wall",&p_->wall,YG_NET_BOOL);
	addNetKey("floor",&p_->floor,YG_NET_BOOL);
	}

staticObject::~staticObject(void)
	{
	delete p_;
	}


void staticObject::reset(void)
	{
	p_->isectMask = YG_ISECT_ALLMASK;
	if (p_->obj)
		{
		p_->obj->setTravMask(PFTRAV_ISECT, p_->isectMask,
					PFTRAV_SELF|PFTRAV_DESCEND,
					PF_SET);
		if ((p_->drawn) && (p_->showLocal))
			p_->obj->setTravMask(PFTRAV_DRAW, 0xffffffff,
					PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		else
			p_->obj->setTravMask(PFTRAV_DRAW, 0,
					PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		}
	ygNode::reset();
	}


void staticObject::message(const ygMessage& msg)
	{
	if (msg == "file")
		{
		if (msg.args.size() == 1)
			loadFile(msg.args[0]);
		else
			msg.error(name(),"(wrong number of arguments)");
		}
	else if (msg == "wall")
		{
		if (msg.args.size() > 0)
			setWall(msg.boolArg(0));
		else
			setWall();
		}
	else if (msg == "floor")
		{
		if (msg.args.size() > 0)
			setFloor(msg.boolArg(0));
		else
			setFloor();
		}
	else if (msg == "draw")
		{
		if (msg.args.size() > 0)
			setDraw(msg.boolArg(0));
		else
			setDraw();
		}
	else if (msg == "showLocal")
		{
		if (msg.args.size() > 0)
			showLocal(msg.boolArg(0));
		else
			showLocal();
		}
	else if (msg == "cache")
		{
		if (msg.args.size() > 0)
			setCache(msg.boolArg(0));
		else
			setCache();
		}
	else
		ygNode::message(msg);
	}


void staticObject::loadFile(const ygString& file)
	{
	if (p_->filename == file)
		return;
	if (p_->obj)
		pfnode()->removeChild(p_->obj);
	if (p_->cache)
		p_->obj = ygPFObjectCache::objectReference(file.c_str());
	else
		p_->obj = pfdLoadFile(file.c_str());
	if (p_->obj)
		{
		p_->filename = file;
		pfnode()->addChild(p_->obj);
		if ((p_->drawn) && (p_->showLocal))
			p_->obj->setTravMask(PFTRAV_DRAW, 0xffffffff,
					PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		else
			p_->obj->setTravMask(PFTRAV_DRAW, 0,
					PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		p_->obj->setTravMask(PFTRAV_ISECT, p_->isectMask,
					PFTRAV_SELF|PFTRAV_DESCEND|PFTRAV_IS_CACHE, PF_SET);
		p_->netFilename = file;
		netKeyChanged("file");
		}
	}


void staticObject::setWall(bool val)
	{
	if (val)
		p_->isectMask |= YG_ISECT_WALLMASK;
	else
		p_->isectMask &= ~YG_ISECT_WALLMASK;
	if (p_->obj)
		p_->obj->setTravMask(PFTRAV_ISECT, p_->isectMask,
				PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
	p_->wall = val;
	netKeyChanged("wall");
	}


void staticObject::setFloor(bool val)
	{
	if (val)
		p_->isectMask |= YG_ISECT_FLOORMASK;
	else
		p_->isectMask &= ~YG_ISECT_FLOORMASK;
	if (p_->obj)
		p_->obj->setTravMask(PFTRAV_ISECT, p_->isectMask,
				PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
	p_->floor = val;
	netKeyChanged("floor");
	}


void staticObject::setDraw(bool val)
	{
	p_->drawn = val;
	netKeyChanged("draw");
	if (p_->obj)
		{
		if (p_->drawn)
			p_->obj->setTravMask(PFTRAV_DRAW, 0xffffffff,
					PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		else
			p_->obj->setTravMask(PFTRAV_DRAW, 0,
					PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		}
	}


void staticObject::showLocal(bool show)
	{
	p_->showLocal = show;
	if (p_->obj)
		{
		if (show)
			p_->obj->setTravMask(PFTRAV_DRAW, 0xffffffff, PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		else
			p_->obj->setTravMask(PFTRAV_DRAW, 0, PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		}
	}


void staticObject::setCache(bool v)
	{
	p_->cache = v;
	}


void staticObject::acceptNetKey(const ygString& key)
	{
	if (key == "file")
		loadFile(p_->netFilename);
	else if (key == "draw")
		setDraw(p_->drawn);
	else if (key == "wall")
		setWall(p_->wall);
	else if (key == "floor")
		setFloor(p_->floor);
	else
		ygNode::acceptNetKey(key);
	}
