#include "ygWorld.h"
#include "turner.h"

extern "C" ygNode* construct_turner(const char* n) { return new turner(n); }

struct _turnerPrivateData
	{
	bool myDebug;
	};

enum _states { TURNER_START, TURNER_GO_FORWARD, TURNER_GO_BACK, TURNER_END };

enum _swingStates { ON, OFF };


turner::turner(const char* n) : ygTransform(n)
	{
	setClassName("turner");
	p_ = new struct _turnerPrivateData;
//	debugFlag("myDebug", &p_->myDebug);
	
	axis_.set(0, 0, 1);
	minAngle_ = 0;
	maxAngle_ = 180;
	duration_ = 1;
	state_ = TURNER_START;
	swingState_ = OFF;
	}

turner::~turner(void)
	{
	delete p_;
	}


void turner::reset(void)
	{
	ygTransform::reset();
	}


void turner::message(const ygMessage& msg)
	{
	if (msg == "axis")
		{
		msg.getVec3Args(axis_);
		initialRotation();
		}
	else if (msg == "minangle")
		{
		minAngle_ = msg.floatArg(0);
		initialRotation();
		}
	else if (msg == "maxangle")
		maxAngle_ = msg.floatArg(0);
	else if (msg == "duration")
		duration_ = msg.floatArg(0);
	else if (msg == "turn")
		turn();
	else if (msg == "turnback")
		turnBack();
	else if (msg == "swing")
		swing();
	else
		ygTransform::message(msg);
	}


void turner::initialRotation(void)
	{
	pfMatrix rotmat;
	rotmat.makeRot( minAngle_, axis_[0], axis_[1], axis_[2] );
	setMatrix(rotmat);
	}

void turner::swing(void)
{
    swingState_ = ON;
    state_ = TURNER_GO_FORWARD;
    startTime_ = ygWorld::FrameTime;
}


void turner::turn(void)
{
    if (state_ == TURNER_START)
    {
        state_ = TURNER_GO_FORWARD;
        startTime_ = ygWorld::FrameTime;
    }
    else if (state_ == TURNER_GO_BACK)
    {
        state_ = TURNER_GO_FORWARD;
        startTime_ = ygWorld::FrameTime -
		(duration_ - (ygWorld::FrameTime -  startTime_));
    }
}


void turner::turnBack(void)
{
    if (state_ == TURNER_END)
    {
        state_ = TURNER_GO_BACK;
        startTime_ = ygWorld::FrameTime;
    }
    else if (state_ == TURNER_GO_FORWARD)
    {
        state_ = TURNER_GO_BACK;
        startTime_ = ygWorld::FrameTime - (duration_ - (ygWorld::FrameTime -  startTime_));
    }
}


void turner::app(void)
	{
	if (state_ == TURNER_GO_FORWARD)
	{
	    float    t = ygWorld::FrameTime -startTime_, angle;
	    pfMatrix rotmat;
    
	    if (t > duration_)
	    {
		t = duration_;
		if (swingState_ == ON)
		{
		    startTime_ = ygWorld::FrameTime;
		    state_ = TURNER_GO_BACK;
		}
		else
		    state_ = TURNER_END;
	    }
    
	    angle = minAngle_ + (maxAngle_ - minAngle_) * t / duration_;
	    rotmat.makeRot(angle, axis_[0], axis_[1], axis_[2]);
	    setMatrix(rotmat);
	}
	else if (state_ == TURNER_GO_BACK)
	{
	    float    t = ygWorld::FrameTime -startTime_, angle;
	    pfMatrix rotmat;
    
	    if (t > duration_)
	    {
		t = duration_;
		if (swingState_ == ON)
		{
		    startTime_ = ygWorld::FrameTime;
		    state_ = TURNER_GO_FORWARD;
		}
		else
		    state_ = TURNER_START;
	    }
    
	    angle = maxAngle_ + (minAngle_ - maxAngle_) * t / duration_;
	    rotmat.makeRot(angle, axis_[0], axis_[1], axis_[2]);
	    setMatrix(rotmat);
	}
	ygTransform::app();
	}
