#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <Performer/pf/pfGroup.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pr/pfGeoSet.h>
#include <Performer/pr/pfTexture.h>
#include <ygWorld.h>
#include "unroller.h"

using namespace std;

extern "C" ygNode* construct_unroller(const char* name,bool master) { return new unroller(name,master); }

struct _unrollerPrivateData
	{
//	bool myDebug;
	};

 pfGeode * createGeode(pfVec3 *verts,pfVec3 *norms,pfVec4 *colors, pfVec2*, int numVerts, pfGeoState *gstate=NULL);


unroller::unroller(const char* name,bool master) : ygNode(name,master)
	{
	setClassName("unroller");
	p_ = new struct _unrollerPrivateData;
//	debugFlag("myDebug", &p_->myDebug);
	 
	 state_ = DO_NOTHING;
	 previousState_ = ROLL_UP;
	 startTime_ = 0;
	 duration_ = 1;
	 
	 xStartSize_ = xEndSize_ = 1;
	 zStartSize_ = zEndSize_ = 1;
	 xTexCoord_ =  yTexCoord_= 1;
	 
//variables for geoset	 
	 pfVec3 *norms;
	 pfVec4 *colors;
	 pfTexture * tex;
	 pfGeode *geode;
	 int numVerts = 4;
	 
	 verts_ = (pfVec3*) pfMalloc(numVerts * sizeof(pfVec3), pfGetSharedArena());
	 norms = (pfVec3*) pfMalloc(numVerts * sizeof(pfVec3), pfGetSharedArena());
	 colors = (pfVec4*) pfMalloc(numVerts * sizeof(pfVec4), pfGetSharedArena());
	 texcoords_ = (pfVec2*) pfMalloc(numVerts * sizeof(pfVec2), pfGetSharedArena());
	 
	 //reversed order of verts to reverse backface culling
	 
	 verts_[0].set(xStartSize_*0.5, 0, 0);
	 verts_[1].set(xStartSize_*0.5, 0, zStartSize_);
	 verts_[2].set(-xStartSize_*0.5, 0, zStartSize_);
	 verts_[3].set(-xStartSize_*0.5, 0, 0);
	  
	
	 texcoords_[0].set(xTexCoord_, 0);
	 texcoords_[1].set(xTexCoord_, yTexCoord_);
	 texcoords_[2].set(0, yTexCoord_);
	 texcoords_[3].set(0, 0);
	 
	 norms[0].set(0, -1, 0);
	 norms[1].set(0, -1, 0);
	 norms[2].set(0, -1, 0);
	 norms[3].set(0, -1, 0);
		 
	 //all colors to 1 = white and we see texture
	 colors[0].set(1, 1, 1, 1);
	 colors[1].set(1, 1, 1, 1);
	 colors[2].set(1, 1, 1, 1);
	 colors[3].set(1, 1, 1, 1);
	 
	 gstate_ = new pfGeoState;
	 tex = new pfTexture;
	 tex->loadFile("map.rgb");
	 gstate_->setAttr(PFSTATE_TEXTURE, tex);
	 gstate_->setMode(PFSTATE_ENTEXTURE, PF_ON);
	 //lighting turned off
	 gstate_->setMode(PFSTATE_ENLIGHTING, PF_OFF);
	 geode = createGeode(verts_, norms, colors, texcoords_, numVerts,  gstate_);
	 pfnode()->addChild(geode);
	}

pfGeode * createGeode(pfVec3 *verts,  pfVec3 *norms, pfVec4 *colors, 
			pfVec2 *texc, int numVerts,  
			pfGeoState *gstate)
	 {
	 int *lengths;
	 pfGeode *geode;
	 pfGeoSet *gset;
	 gset = new pfGeoSet;
	 gset->setPrimType(PFGS_QUADS);
	 gset->setNumPrims(1);
	 
	 lengths = (int *) pfMalloc(sizeof(int),pfGetSharedArena());
	 lengths[0] = numVerts;
	 gset->setPrimLengths(lengths);
	 gset->setAttr(PFGS_COORD3, PFGS_PER_VERTEX, verts, NULL);
	 gset->setAttr(PFGS_NORMAL3, PFGS_PER_VERTEX, norms, NULL);
	 gset->setAttr(PFGS_COLOR4, PFGS_PER_VERTEX, colors, NULL);
	 gset->setAttr(PFGS_TEXCOORD2, PFGS_PER_VERTEX, texc, NULL);
	 if (gstate)
		gset->setGState(gstate);
	 else
		gset->setGState(new pfGeoState);
	 geode = new pfGeode;
	 geode->addGSet(gset);
	 return geode;
	}

unroller::~unroller(void)
	{
	delete p_;
	}

void unroller::reset(void)
	{
	ygNode::reset();
	}


void unroller::message(const ygMessage& msg)
	{
	if (msg == "toggle")
	    setRollState();
	else if (msg == "startSize")
	    setStartSize(msg);
	else if (msg == "endSize")
	    setEndSize(msg);
	else if (msg == "period")
	    duration_ = msg.floatArg(0);
	
	else
		ygNode::message(msg);
	}

void unroller::setStartSize(const ygMessage& msg)
	{
	 xStartSize_ = msg.floatArg(0);
	 zStartSize_ = msg.floatArg(1);
	 setVerts(xStartSize_,  zStartSize_);
	}
	
void unroller::setEndSize(const ygMessage& msg)
	{
	xEndSize_ = msg.floatArg(0);
	zEndSize_ = msg.floatArg(1);
	}
	
void unroller::setVerts( float sizeX, float sizeZ)
	{
	verts_[0].set(sizeX*0.5, 0, 0);
	verts_[1].set(sizeX*0.5, 0, sizeZ);
	verts_[2].set(-sizeX*0.5, 0, sizeZ);
	verts_[3].set(-sizeX*0.5, 0, 0);
	setTexCoords((sizeX/xEndSize_),(sizeZ/zEndSize_)); 
	}

void unroller::setTexCoords( float texX, float texZ)
	{
	 texcoords_[0].set(texX, 0);
	 texcoords_[1].set(texX, texZ);
	 texcoords_[2].set(0, texZ);
	 texcoords_[3].set(0, 0);
	}	
	

void unroller::setRollState(void)
{   
    if (previousState_ == UN_ROLL)
    {
	state_ = ROLL_UP;
	eventOccurred("rollup");
    }
    else
    {
	state_ = UN_ROLL;
	eventOccurred("unroll");
    }
	
    startTime_ = ygWorld::FrameTime;
    
}

void unroller::app(void)
	{
//need another doNothing state
	if(state_ == DO_NOTHING)
	    {
	    //do nothing
	    }
	    
	else if(state_ == UN_ROLL)
	    {
	    float cur_sizeX;
	    float cur_sizeZ;
	    float a = 1.0;
	    
	    if (((ygWorld::FrameTime - startTime_) < duration_) && (fabs(duration_) > 0.0000001f))
		{
		a = (ygWorld::FrameTime  - startTime_) / duration_;
		}
	    
	    else
		{
		state_ = DO_NOTHING;
		previousState_ = UN_ROLL;
		}
	    cur_sizeX = (1.0-a)*xStartSize_ + a*xEndSize_;
	    cur_sizeZ = (1.0-a)*zStartSize_ + a*zEndSize_;
	    setVerts(cur_sizeX, cur_sizeZ);
	    }
	    
	else if(state_ == ROLL_UP)
	    {
	    float cur_sizeX;
	    float cur_sizeZ;
	    float a = 1.0;
	    
	    if (((ygWorld::FrameTime - startTime_) < duration_) && (fabs(duration_) > 0.0000001f))
		{
		a = (ygWorld::FrameTime  - startTime_) / duration_;
		}
	    
	    else
		{
		state_ = DO_NOTHING;
		previousState_ = ROLL_UP;
		}
	    cur_sizeX = (1.0-a)*xEndSize_ + a*xStartSize_;
	    cur_sizeZ = (1.0-a)*zEndSize_ + a*zStartSize_;
	    setVerts(cur_sizeX, cur_sizeZ);
	    
	    }
	
	ygNode::app();
	}
