#include <stdlib.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <CAVERN.hxx>
#include "ygNetClient.h"
#include "ygKeyDatabase.h"

using namespace std;


void parseCommandLine(int argc,char **argv);
int udpIntercept(CAVERNnet_udp_c *,char**,int *,void *);
void saveKeyData(ygKeyDatabase *db,CAVERNnet_datapack_c& packer, char *buffer);
int tcpIntercept(CAVERNnet_tcpReflectorClient_c *client, char** buffer,
		int *bufferSize, void *userData);
void dualLoop(ygKeyDatabase *db);
void * tcpDownload(void *userData);

#define MAX_LINKS 256

char * linkHost[MAX_LINKS];
int numLinks = 0;
int portNumber = 3500;
char udpBuffer[YG_NET_MAX_BUFFER_SIZE];
char tcpBuffer[YG_NET_MAX_BUFFER_SIZE];
ygKeyDatabase *ygdb;

main(int argc, char **argv)
	{
	cout << "ygrepeater for Ygdrasil version 0.1.4\n";
	parseCommandLine(argc,argv);
	CAVERNinit();
	ygdb = new ygKeyDatabase;
	dualLoop(ygdb);
	}


/**************************************************************************/

void dualLoop(ygKeyDatabase *db)
	{
	FILE *dumpFP;
	int prevTime=time(NULL), nowTime, sprevTime;
	int dumpInterval = 0, statsInterval = 0;
	int i;
	CAVERNnet_udpReflector_c *udpReflector;
	udpReflector = new CAVERNnet_udpReflector_c;
	udpReflector->setIncomingPort(portNumber);
	for (i=0; i < numLinks; i++)
		udpReflector->setForcedDestination(linkHost[i], portNumber);
	udpReflector->intercept(udpIntercept,db);
	udpReflector->init();
	if (getenv("YGREPEATER_DUMP_INTERVAL"))
		{
		dumpInterval = atoi(getenv("YGREPEATER_DUMP_INTERVAL"));
		char filename[256];
		sprintf(filename, "ygdbdump.%d", getpid());
		dumpFP = fopen(filename, "w");
		if (!dumpFP)
			dumpFP = stdout;
		}
	if (getenv("YGREPEATER_STATS_INTERVAL"))
		statsInterval = atoi(getenv("YGREPEATER_STATS_INTERVAL"));
	CAVERNnet_tcpReflector_c *tcpReflector;
	tcpReflector = new CAVERNnet_tcpReflector_c;
	tcpReflector->init(portNumber);
	for (i=0; i < numLinks; i++)
		tcpReflector->setForcedDestination(linkHost[i], portNumber);
	tcpReflector->intercept(tcpIntercept,db);
	signal(SIGSEGV, SIG_DFL);
	signal(SIGBUS, SIG_DFL);
	while (true)
		{
		udpReflector->process();
		tcpReflector->process();
		if (dumpInterval > 0)
			{
			nowTime = time(NULL);
			if (nowTime - prevTime >= dumpInterval)
				{
				db->printContents(dumpFP);
				prevTime = nowTime;
				}
			}
		if (statsInterval > 0)
			{
			nowTime = time(NULL);
			if (nowTime - sprevTime >= statsInterval)
				{
				db->printStats();
				sprevTime = nowTime;
				}
			}
		}
	}



/**************************************************************************/


int udpIntercept(CAVERNnet_udp_c *client, char** buffer, int *bufferSize,
		void *userData)
	{
	CAVERNnet_datapack_c packer;
	int packetType;
	ygKeyDatabase *db = (ygKeyDatabase *) userData;
	packer.initUnpack(*buffer,*bufferSize);
	packer.unpackInt(&packetType);
	if ((packetType == YG_NET_PUT_KEY) ||
	    (packetType == YG_NET_REFRESH_KEY))
		saveKeyData(db,packer, udpBuffer);
	return CAVERNnet_udpReflector_c::OK;
	}


/**************************************************************************/


#if 1
int tcpIntercept(CAVERNnet_tcpReflectorClient_c *client, char** buffer,
		int *bufferSize, void *userData)
	{
	ygKeyDatabase *db = (ygKeyDatabase *) userData;
	CAVERNnet_datapack_c packer;
	int packetType;
	packer.initUnpack(*buffer,*bufferSize);
	packer.unpackInt(&packetType);
	if (packetType == YG_NET_REQUEST_ALL)
		{
		printf("sending database to new client\n");
		db->sendContents(client);
		printf("done sending database\n");
		fflush(stdout);
		return CAVERNnet_tcpReflector_c::SKIP_DISTRIBUTION;
		}
	else if ((packetType == YG_NET_PUT_KEY) ||
		 (packetType == YG_NET_REFRESH_KEY))
		saveKeyData(db, packer, tcpBuffer);
	return CAVERNnet_tcpReflector_c::OK;
	}
#else
int tcpIntercept(CAVERNnet_tcpReflectorClient_c *client, char** buffer,
		int *bufferSize, void *userData)
	{
	CAVERNnet_datapack_c packer;
	int packetType;
	packer.initUnpack(*buffer,*bufferSize);
	packer.unpackInt(&packetType);
	if (packetType == YG_NET_REQUEST_ALL)
		{
		CAVERNts_thread_c tcpThread;
		tcpThread.create(tcpDownload,client);
		return CAVERNnet_tcpReflector_c::SKIP_DISTRIBUTION;
		}
	else if ((packetType == YG_NET_PUT_KEY) ||
		 (packetType == YG_NET_REFRESH_KEY))
		saveKeyData(ygdb, packer, tcpBuffer);
	return CAVERNnet_tcpReflector_c::OK;
	}
#endif

void * tcpDownload(void *userData)
	{
	CAVERNnet_tcpReflectorClient_c *client = (CAVERNnet_tcpReflectorClient_c *)userData;
	ygdb->sendContents(client);
	return 0;
	}


/**************************************************************************/



void saveKeyData(ygKeyDatabase *db,CAVERNnet_datapack_c& packer, char *buffer)
	{
	ygString path, key;
	int length;
/* Get path string length & data */
	packer.unpackInt(&length);
	packer.unpack(buffer, length);
	path.append(buffer);
/* Get key string length & data */
	packer.unpackInt(&length);
	packer.unpack(buffer, length);
	key.append(buffer);
/* Get data length & data */
	packer.unpackInt(&length);
	packer.unpack(buffer, length);
	db->store(path, key, buffer, length);
	}


void parseCommandLine(int argc,char **argv)
	{
	int i;
	for (i=1; i < argc; i++)
		{
		if (!strncmp(argv[i],"--",2))
			{
			if (!strcasecmp(argv[i],"--link"))
				{
				i++;
				if (numLinks < MAX_LINKS)
					{
					linkHost[numLinks] = strdup(argv[i]);
					numLinks++;
					}
				else
					fprintf(stderr, "ERROR: cannot link to more than"
						" %d hosts; link to '%s' ignored\n",
						MAX_LINKS, argv[i]);
				}
			else if (!strcasecmp(argv[i],"--port"))
				{
				i++;
				portNumber = atoi(argv[i]);
				}
			else
				fprintf(stderr,"WARNING: unknown command line "
					"option '%s'\n",argv[i]);
			}
		}
	}
