#include <stdio.h>
#include <vector>
#include <set>
#include "ygString.h"
#include "xptTokenStream.h"

using namespace std;

struct xpoption
	{
	ygString tag;
	ygString val;
	};

void translateNode(const ygString& token, xptTokenStream * tokenStream,
			FILE *outfp);
void translateSceneNode(const ygString& name,vector<xpoption*>& optList,
			FILE *outfp);
xpoption* createOption(const ygString& tag,const ygString& val);
ygString translateRecipient(const ygString& name);



main(int argc,char **argv)
	{
	xptTokenStream *tokenStream;
	ygString token;
	bool haveParent;
	FILE *outfp;
	set<ygString> nodeTypesSeen;
	if (argc != 3)
		{
		fprintf(stderr,"Usage: %s input-XP-file output-YG-file\n",
			argv[0]);
		exit(-1);
		}
	outfp = fopen(argv[2],"w");
	if (!outfp)
		{
		perror(argv[2]);
		exit(-1);
		}
	tokenStream = new xptTokenStream(argv[1],outfp);
	haveParent = false;
	while (tokenStream->next(token))
		{
		if (token == "{")
			{
			if (!haveParent)
				fprintf(outfp,"group () ");
			fprintf(outfp,"{");
			haveParent = false;
			}
		else if (token == "}")
			{
			fprintf(outfp,"}");
			haveParent = false;
			}
		else
			{
			nodeTypesSeen.insert(token);
			translateNode(token, tokenStream, outfp);
			haveParent = true;
			}
		}
	fclose(outfp);
	cout << "Node types seen:\n ";
	copy(nodeTypesSeen.begin(), nodeTypesSeen.end(),
		ostream_iterator<ygString>(cout, " "));
	cout << endl;
	}


void translateNode(const ygString& nodeType, xptTokenStream * tokenStream,
			FILE *outfp)
	{
	vector<xpoption*> optList;
	xpoption* opt;
	ygString token, tag, nodeName;
	int i;
	if ((tokenStream->next(token,false)) && (token == "(") &&
	    (tokenStream->next(token,false)))
		{
		while (token != ")")
			{
			tag = token;
			if (!tokenStream->next(token,false)) break;
			if (token != "=")
				{
				fprintf(stderr,"ERROR: missing = in arguments"
					" for \"%s\" (line %d)\n",
					nodeType.c_str(),
					tokenStream->linenumber());
				continue;
				}
			if (!tokenStream->next(token,false)) break;
			if (tag == "name")
				nodeName = token;
			else
				{
				opt = createOption(tag,token);
				optList.push_back(opt);
				}
			if (!tokenStream->next(token,false)) break;
			if (token == ",")
				if (!tokenStream->next(token,false)) break;
			}
		}
	if (nodeType == "scene")
		translateSceneNode(nodeName,optList,outfp);
	else
		{
		fprintf(outfp, "%s", nodeType.c_str());
		if (nodeName.length() > 0)
			fprintf(outfp," %s",nodeName.c_str());
		fprintf(outfp," (");
		for (i=0; i < optList.size(); i++)
			{
			fprintf(outfp,"%s",optList[i]->tag.c_str());
			if (optList[i]->val.length() > 0)
				fprintf(outfp,"(%s)",optList[i]->val.c_str());
			if (i < optList.size()-1)
				fprintf(outfp,", ");
			}
		fprintf(outfp,")");
		}
	}


void translateSceneNode(const ygString& name,vector<xpoption*>& optList,
			FILE *outfp)
	{
	ygString file;
	vector<xpoption*>::iterator iter;
	int i;
	for (iter=optList.begin(); iter != optList.end(); ++iter)
		{
		if ((*iter)->tag == "file")
			{
			file = (*iter)->val;
			optList.erase(iter);
			break;
			}
		}
	fprintf(outfp,"switch ");
	if (name.length() > 0)
		fprintf(outfp," %s",name.c_str());
	fprintf(outfp," (");
	for (i=0; i < optList.size(); i++)
		{
		fprintf(outfp,"%s",optList[i]->tag.c_str());
		if (optList[i]->val.length() > 0)
			fprintf(outfp,"(%s)",optList[i]->val.c_str());
		if (i < optList.size()-1)
			fprintf(outfp,", ");
		}
	fprintf(outfp,")");
	if (file.length() > 0)
		fprintf(outfp," {\n#include \"%s\"\n}\n", file.c_str());
	}


#define SKIP_COMMA_AND_WHITESPACE(s,i)  \
                { if (i<s.length()) i = s.find_first_not_of(" \t",i+1); }

xpoption* createOption(const ygString& tag,const ygString& val)
	{
	xpoption * opt = new xpoption;
	opt->val = val;
	if (tag == "translate")
		opt->tag = "position";
	else if (tag == "rotate")
		opt->tag = "orientation";
	else if (tag == "scale")
		opt->tag = "size";
	else if (tag == "eventMessage")
		{
		ygString * s;
		int index=0;
		opt->tag = "when";
		s = val.nextToken(",",&index);
		if (!s) return opt;
		opt->val = *s;
		opt->val += ",";
		SKIP_COMMA_AND_WHITESPACE(val,index)
		s = val.nextToken(",",&index);
		if (!s) return opt;
		opt->val += translateRecipient(*s);
		opt->val += ".";
		SKIP_COMMA_AND_WHITESPACE(val,index)
		s = val.nextToken(",",&index);
		if (!s) return opt;
		{ int space = s->find_first_of(" \t",0);
		if (space < s->length())
			{
			s->replace(space,1,"(");
			(*s) += ')';
			}
		}
		opt->val += *s;
		SKIP_COMMA_AND_WHITESPACE(val,index)
		s = val.nextToken(",",&index);
		if (s)
			{
			opt->val += "+";
			opt->val += *s;
			}
		}
	else if ((tag == "initVal") && ((val == "1") || (val == "on")))
		{
		opt->tag = "on";
		opt->val = "";
		}
	else if ((tag == "initVal") && ((val == "0") || (val == "off")))
		{
		opt->tag = "off";
		opt->val = "";
		}
	else if ((tag == "initState") && ((val == "1") || (val == "on")))
		{
		opt->tag = "enable";
		opt->val = "";
		}
	else if ((tag == "initState") && ((val == "0") || (val == "off")))
		{
		opt->tag = "disable";
		opt->val = "";
		}
	else if (tag == "maxDistance")
		opt->tag = "falloffDistance";
	else if (tag == "initChild")
		opt->tag = "select";
	else if (tag == "box")
		{
		opt->tag = "volume";
		opt->val.insert(0,"box ");
		}
	else if (tag == "sphere")
		{
		opt->tag = "volume";
		opt->val.insert(0,"sphere ");
		}
	else
		opt->tag = tag;
	return opt;
	}


ygString translateRecipient(const ygString& name)
	{
	if (name == "%navigator")
		return "$user";
	return name;
	}
