#include <Performer/pr/pfLinMath.h>
#include <Performer/pf/pfScene.h>
#include <pfcave.h>
#include "ygWorld.h"
#include "ygUtil.h"
#include "ygSphereIsect.h"
#include "ygCAVENavigator.h"

using namespace std;

extern "C" ygNode* construct_ygCAVENavigator(const char* name,bool master) { return new ygCAVENavigator(name,master); }


ygCAVENavigator::ygCAVENavigator(const char* name,bool master) : ygNavigator(name,master)
	{
	rotSpeed_ = 45;
	transSpeed_ = 10;
	collide_ = true;
	collideRadius_ = 1;
	fly_ = true;
	}


void ygCAVENavigator::message(const ygMessage& msg)
	{
	if (msg == "teleport")
		{
		pfVec3 pos;
		if (msg.getVec3Args(pos))
			teleportTo(pos);
		}
	else if (msg == "speed")
		{
		if (msg.args.size() > 0)
			setSpeed(msg.floatArg(0));
		}
	else if (msg == "rotspeed")
		{
		if (msg.args.size() > 0)
			setRotSpeed(msg.floatArg(0));
		}
	else if (msg == "collide")
		{
		if (msg.args.size() > 0)
			setCollide(msg.boolArg(0));
		else
			setCollide();
		}
	else if (msg == "collideRadius")
		collideRadius_ = msg.floatArg(0);
	else if (msg == "togglecollide")
		setCollide(!collide_);
	else if (msg == "fly")
		{
		if (msg.args.size() > 0)
			setFly(msg.boolArg(0));
		else
			setFly();
		}
	else if (msg == "togglefly")
		setFly(!fly_);
	else if (msg == "printnav")
		{
		cout << "Navigator position = " << data().position()
			<< "  orientation = " << data().orientation() << endl;
		}
	else
		ygNavigator::message(msg);
	}


void ygCAVENavigator::teleportTo(const pfVec3& pos)
	{
	data().setPosition(pos);
	resetCollision();
	}


void ygCAVENavigator::setSpeed(float s)
	{
	transSpeed_ = s;
	}


void ygCAVENavigator::setRotSpeed(float s)
	{
	rotSpeed_ = s;
	}


void ygCAVENavigator::setCollide(bool val)
	{
	if ((val) && (!collide_))
		resetCollision();
	collide_ = val;
	}


void ygCAVENavigator::setFly(bool val)
	{
	fly_ = val;
	}


void ygCAVENavigator::app(void)
	{
	if (fabs(CAVE_JOYSTICK_X) > 0.2f)
		{
		float h = data().heading();
		h -= CAVE_JOYSTICK_X * (fabs(CAVE_JOYSTICK_X)-0.2f) *
			rotSpeed_ * ygWorld::FrameDeltaTime;
		data().setHeading(h);
		}
	if (fabs(CAVE_JOYSTICK_Y) > 0.1f)
		{
		pfVec3 dir, pos = data().position();
		CAVEGetVector(CAVE_WAND_FRONT_NAV, dir.vec);
		if (!fly_)
			dir[PF_Z] = 0;
		pos += dir * CAVE_JOYSTICK_Y * transSpeed_
			* ygWorld::FrameDeltaTime;
		data().setPosition(pos);
		}
	if (!fly_)
		checkGround();
	if (collide_)
		checkCollision();
	ygNavigator::app();
	}


void ygCAVENavigator::checkGround(void)
	{
	pfSegSet segset;
	pfHit **hits[32];
	segset.activeMask = 1;
	segset.isectMask = YG_ISECT_FLOORMASK;
	segset.discFunc = NULL;
	segset.bound = NULL;
	segset.mode = PFTRAV_IS_PRIM;
	segset.segs[0].dir.set(0.0f, 0.0f, -1.0f);
	segset.segs[0].length = 5000.0f;
	CAVEGetPosition(CAVE_HEAD_NAV, segset.segs[0].pos.vec);
	if (ygWorld::World->pfscene()->isect(&segset, hits))
		{
		pfVec3 pnt, xpnt, pos = data().position();
		pfMatrix xmat;
		(*hits[0])->query(PFQHIT_POINT, pnt.vec);
		(*hits[0])->query(PFQHIT_XFORM, (float*)xmat.mat);
		xpnt.xformPt(pnt, xmat);
		if (pos[PF_Z] > xpnt[PF_Z] + 2.0f)
			pos[PF_Z] -= 2.0f;
		else
			pos[PF_Z] = xpnt[PF_Z];
		data().setPosition(pos);
		}
	}


void ygCAVENavigator::checkCollision(void)
	{
	pfVec3 headTrackerPos, rotHeadTrackerPos, curPos;
	pfMatrix rotXform;
	bool OK;
	CAVEGetPosition(CAVE_HEAD, headTrackerPos.vec);
	rotXform.makeRot(data().heading(),0,0,1);
	rotHeadTrackerPos.xformPt(headTrackerPos, rotXform);
	curPos = rotHeadTrackerPos + data().position();
	if (curPos.almostEqual(prevCollidePos_,.001))
		return;
	OK = positionOK(curPos);
	if (OK)
		prevCollidePos_ = curPos;
	else
		data().setPosition(prevCollidePos_ - rotHeadTrackerPos);
	}


bool ygCAVENavigator::positionOK(pfVec3 pos)
	{
	pfSphere sphere;
	sphere.center = pos;
	sphere.radius = collideRadius_;
	return !ygSphereIsect(ygWorld::World->pfscene(),&sphere,YG_ISECT_WALLMASK);
	}


void ygCAVENavigator::resetCollision(void)
	{
	pfVec3 headTrackerPos, rotHeadTrackerPos;
	pfMatrix rotXform;
	CAVEGetPosition(CAVE_HEAD, headTrackerPos.vec);
	rotXform.makeRot(data().heading(),0,0,1);
	rotHeadTrackerPos.xformPt(headTrackerPos, rotXform);
	prevCollidePos_ = rotHeadTrackerPos + data().position();
	}
