#include <stdio.h>
#include <iostream>
#include <dirent.h>
#include <strings.h>
#include <dlfcn.h>
#include <map>
#include "ygDebugFlags.h"
#include "ygDSOLoader.h"

using namespace std;


typedef struct _ygDSOInfo
	{
	ygString name;
	ygString fullpath;
	bool loaded;
	bool loadFailed;	/* For some fucked up reason, Linux's dlopen will
				   succeed the second time you try a dso, even though
				   dependencies still aren't met; as a result, the
				   constructor quickly crashes.  This flag makes sure
				   we only try to load once */
	ygNodeConstructorPtr constructPtr;
	void *dlhandle;
	} ygDSOInfo;


struct _ygDSOLoaderPrivateData
	{
	map<ygString,ygDSOInfo*> infomap;
	bool debugLoad;
	};


struct _ygDSOLoaderPrivateData * ygDSOLoader::p_ = NULL;


void ygDSOLoader::init(void)
	{
	if (!p_)
		p_ = new struct _ygDSOLoaderPrivateData;
	p_->debugLoad = ygDebugFlags::checkDebugEnv("ygdso.load");
	searchForDSOs();
	if (getenv("YG_PRELOAD_CLASSES"))
		preLoadClasses(getenv("YG_PRELOAD_CLASSES"));
	}


void ygDSOLoader::preLoadClasses(const ygString &classes)
	{
	int i=0;
	ygString * name;
	while (name = classes.nextToken(" \t",&i))
		{
		getNodeConstructor(*name);
		delete name;
		}
	}


void ygDSOLoader::searchForDSOs(void)
	{
	searchDirectory(".");
	if (getenv("YG_DSO_PATH"))
		{
		ygString searchPath(getenv("YG_DSO_PATH"));
		ygString *dir;
		int i=0;
		while (dir = searchPath.nextToken(":",&i))
			{
			searchDirectory(*dir);
			delete dir;
			}
		}
	}


void ygDSOLoader::searchDirectory(const ygString& dirname)
	{
	DIR *dir = opendir(dirname.c_str());
	if (dir)
		{
		struct dirent * ent;
		while (ent = readdir(dir))
			{
			int len = strlen(ent->d_name);
			if (strcmp(ent->d_name+len-3, ".so") == 0)
				{
				ygDSOInfo *info = new ygDSOInfo;
				info->name = ent->d_name;
				info->fullpath = dirname;
				info->fullpath += '/';
				info->fullpath += info->name;
				info->loaded = false;
				info->loadFailed = false;
				info->constructPtr = NULL;
				info->name.erase(info->name.length()-3);
				p_->infomap[info->name] = info;
				}
			}
		closedir(dir);
		}
	}


ygNodeConstructorPtr ygDSOLoader::getNodeConstructor(const ygString& nodeType)
	{
	ygDSOInfo *info = p_->infomap[nodeType];
	if (info)
		{
		if (!info->loaded)
			loadDSO(info);
		return info->constructPtr;
		}
	else
		{
		cerr << "ERROR: cannot find DSO for node type '"
			<< nodeType << "'\n";
		return NULL;
		}
	}


bool ygDSOLoader::loadDSO(ygDSOInfo *info)
	{
	if ((!info) || (info->loadFailed))
		return false;
	if (p_->debugLoad)
		cout << "ygDSOLoader loading DSO for " << info->name << endl;
	if (!loadDSODependencies(info))
		{
		cerr << "ERROR: failed to load dependencies for "
			<< info->fullpath << endl;
		return false;
		}
	info->dlhandle = dlopen(info->fullpath.c_str(), RTLD_LAZY|RTLD_GLOBAL);
	if (!info->dlhandle)
		{
		info->loadFailed = true;
		cerr << "ERROR: failed to open " << info->fullpath << endl;
		cerr << dlerror() << endl;
		return false;
		}
	ygString constructorName("construct_");
	constructorName += info->name;
	info->constructPtr = (ygNodeConstructorPtr)
				dlsym(info->dlhandle,constructorName.c_str());
	info->loaded = true;
	return true;
	}


bool ygDSOLoader::loadDSODependencies(const ygDSOInfo *info)
	{
	FILE *fp;
	ygString filename(info->fullpath);
	filename.erase(filename.length()-2);	/* Remove the "so" */
	filename += "ygdep";
	fp = fopen(filename.c_str(),"r");
	if (fp)
		{
		char buf[256],name[256];
		while (fgets(buf,sizeof(buf),fp))
			{
			if (sscanf(buf,"%s",name) < 1)
				continue;
			ygDSOInfo* info = p_->infomap[ygString(name)];
			if (info)
				{
				if (!loadDSO(info))
					return false;
				}
			else
				{
				cerr << "ERROR: can't find DSO for "
					<< name << endl;
				return false;
				}
			}
		fclose(fp);
		}
	return true;
	}
