#include "ygUtil.h"
#include "ygWorld.h"
#include "ygNetKeys.h"
#include "ygEnvironment.h"

using namespace std;

extern "C" ygNode* construct_ygEnvironment(const char* name,bool net) { return new ygEnvironment(name,net); }

struct _ygEnvironmentPrivateData
	{
	pfVec2 clip;
	pfVec3 skyColor;
	bool fog;
	int fogType;
	pfVec3 fogColor;
	float fogOnset, fogOpaque;
	int lastApply;
	};


ygEnvironment::ygEnvironment(const char* name,bool net) : ygSpace(name,net)
	{
	setClassName("ygEnvironment",true);
	p_ = new struct _ygEnvironmentPrivateData;
	p_->clip[0] = .1;
	p_->clip[1] = 1000;
	p_->skyColor.set(0,0,0);
	p_->fog = false;
	p_->lastApply = -999;
	addNetKey("clip", &p_->clip, YG_NET_VEC2);
	addNetKey("sky", &p_->skyColor, YG_NET_VEC3);
	addNetKey("fog", &p_->fog, YG_NET_BOOL);
	addNetKey("fogType", &p_->fogType, YG_NET_INT);
	addNetKey("fogColor", &p_->fogColor, YG_NET_VEC3);
	addNetKey("fogOnset", &p_->fogOnset, YG_NET_FLOAT);
	addNetKey("fogOpaque", &p_->fogOpaque, YG_NET_FLOAT);
	}

ygEnvironment::~ygEnvironment(void)
	{
	delete p_;
	}


void ygEnvironment::reset(void)
	{
	setClip(.1, 1000);
	setSkyColor(pfVec3(0,0,0));
	ygSpace::reset();
	}


void ygEnvironment::message(const ygMessage& msg)
	{
	if (msg == "clip")
		{
		if (msg.args.size() != 2)
			msg.error(name(),"(wrong number of arguments)");
		else
			setClip(msg.floatArg(0),msg.floatArg(1));
		}
	else if (msg == "skyColor")
		{
		pfVec3 color;
		if (msg.getVec3Args(color))
			setSkyColor(color);
		else
			msg.error(name(),"(wrong number of arguments)");
		}
	else if (msg == "fog")
		{
		if (msg.args.size() < 1)
			msg.error(name(),"(wrong number of arguments)");
		else if (msg.args[0] == "off")
			fogOff();
		else if (msg.args.size() != 6)
			msg.error(name(),"(wrong number of arguments)");
		else
			{
			int fogtype;
			pfVec3 color;
			float onset, opaque;
			if ((msg.args[0] == "lin") || (msg.args[0] == "linear"))
				fogtype = PFFOG_PIX_LIN;
			else if (msg.args[0] == "exp")
				fogtype = PFFOG_PIX_EXP;
			else if (msg.args[0] == "exp2")
				fogtype = PFFOG_PIX_EXP2;
			msg.getVec3Args(color,1);
			onset = msg.floatArg(4);
			opaque = msg.floatArg(5);
			setFog(fogtype,color,onset,opaque);
			}
		}
	else
		ygSpace::message(msg);
	}


void ygEnvironment::setClip(float near,float far)
	{
	p_->clip[0] = near;
	p_->clip[1] = far;
	p_->lastApply = -999;
	netKeyChanged("clip");
	}


float ygEnvironment::nearClip(void) const
	{
	return p_->clip[0];
	}


float ygEnvironment::farClip(void) const
	{
	return p_->clip[1];
	}


void ygEnvironment::setSkyColor(const pfVec3& color)
	{
	p_->skyColor = color;
	p_->lastApply = -999;
	netKeyChanged("sky");
	}


const pfVec3& ygEnvironment::skyColor(void) const
	{
	return p_->skyColor;
	}


void ygEnvironment::fogOff(void)
	{
	p_->fog = false;
	p_->lastApply = -999;
	netKeyChanged("fog");
	}


void ygEnvironment::setFog(int fogtype,const pfVec3& color,float onset,float opaque)
	{
	p_->fog = true;
	p_->fogType = fogtype;
	p_->fogColor = color;
	p_->fogOnset = onset;
	p_->fogOpaque = opaque;
	p_->lastApply = -999;
	netKeyChanged("fog");
	netKeyChanged("fogType");
	netKeyChanged("fogColor");
	netKeyChanged("fogOnset");
	netKeyChanged("fogOpaque");
	}


void ygEnvironment::app(void)
	{
	ygNode * viewer = ygWorld::World->viewer();
	if ((viewer) && (contains(viewer->origin(this))))
		apply();
	ygSpace::app();
	}


void ygEnvironment::apply(void) const
	{
	if (p_->lastApply < ygWorld::FrameNumber-1)
		{
		ygWorld::World->setClip(p_->clip[0], p_->clip[1]);
		ygWorld::World->setSkyColor(p_->skyColor);
		if (p_->fog)
			ygWorld::World->setFog(p_->fogType, p_->fogColor,
						p_->fogOnset, p_->fogOpaque);
		else
			ygWorld::World->fogOff();
		}
	p_->lastApply = ygWorld::FrameNumber;
	}
