#include <Performer/pf/pfLightSource.h>
#include <Performer/pf/pfGroup.h>
#include "ygNetKeys.h"
#include "ygLight.h"

extern "C" ygNode* construct_ygLight(const char* name,bool master) { return new ygLight(name,master); }

struct _ygLightPrivateData
	{
	pfLightSource * light;
	struct
		{
		pfVec4 position;
		pfVec3 diffuse;
		bool on;
		} net;
	bool prevOn;
	};


ygLight::ygLight(const char* name,bool master) : ygNode(name,master)
	{
	setClassName("ygLight",true);
	p_ = new struct _ygLightPrivateData;
	p_->light = new pfLightSource;
	pfnode()->addChild(p_->light);
	addNetKey("pos",&p_->net.position,YG_NET_VEC4);
	addNetKey("dif",&p_->net.diffuse,YG_NET_VEC3);
	addNetKey("on",&p_->net.on,YG_NET_BOOL);
	reset();
	}


ygLight::~ygLight(void)
	{
	pfnode()->removeChild(p_->light);
	pfDelete(p_->light);
	delete p_;
	}


void ygLight::reset(void)
	{
	if (isNetMaster())
		{
		setPosition(0,0,1,0);
		setDiffuse(1,1,1);
		on();
		p_->prevOn = true;
		}
	ygNode::reset();
	}


void ygLight::app(void)
	{
	bool ison = isOn();
	if ((ison) && (!p_->prevOn))
		eventOccurred("LightOn");
	else if ((!ison) && (p_->prevOn))
		eventOccurred("LightOff");
	p_->prevOn = ison;
	ygNode::app();
	}


/*****  LightSource functions *****/

bool ygLight::isOn(void) const
	{
	return (p_->light->isOn() != 0);
	}


void ygLight::on(void)
	{
	if (!isOn())
		{
		p_->light->on();
		p_->net.on = true;
		netKeyChanged("on");
		}
	}


void ygLight::off(void)
	{
	if (isOn())
		{
		p_->light->off();
		p_->net.on = false;
		netKeyChanged("on");
		}
	}


void ygLight::toggle(void)
	{
	if (isOn())
		off();
	else
		on();
	}


void ygLight::setPosition(float x,float y,float z,float w)
	{
	p_->light->setPos(x,y,z,w);
	p_->net.position.set(x,y,z,w);
	netKeyChanged("pos");
	}


void ygLight::setDiffuse(float r,float g,float b)
	{
	p_->light->setColor(PFLT_DIFFUSE, r, g, b);
	p_->net.diffuse.set(r,g,b);
	netKeyChanged("dif");
	}


/***** Message functions *****/

void ygLight::message(const ygMessage& msg)
	{
	if (msg == "on")
		on();
	else if (msg == "off")
		off();
	else if (msg == "toggle")
		toggle();
	else if (msg == "position")
		setPosition(msg);
	else if (msg == "diffuse")
		setDiffuse(msg);
	else
		ygNode::message(msg);
	}


void ygLight::setPosition(const ygMessage& msg)
	{
	pfVec4 v4;
	pfVec3 v3;
	if (msg.getVec4Args(v4))
		setPosition(v4);
	else if (msg.getVec3Args(v3))
		setPosition(v3);
	else
		msg.error(name(),"(wrong number of arguments)");
	}


void ygLight::setDiffuse(const ygMessage& msg)
	{
	pfVec3 v;
	if (msg.getVec3Args(v))
		setDiffuse(v);
	else
		msg.error(name(),"(wrong number of arguments)");
	}


/***** Network functions *****/

void ygLight::acceptNetKey(const ygString& key)
	{
	if (key == "pos")
		setPosition(p_->net.position);
	else if (key == "dif")
		setDiffuse(p_->net.diffuse);
	else if (key == "on")
		{
		if (p_->net.on)
			on();
		else
			off();
		}
	else
		ygNode::acceptNetKey(key);
	}
