#include <map>
#include "ygUtil.h"
#include "ygTokenStream.h"
#include "ygNode.h"
#include "ygTransform.h"
#include "ygSimpleTransform.h"
#include "ygObject.h"
#include "ygLight.h"
#include "ygSwitch.h"
#include "ygSelector.h"
#include "ygUser.h"
#include "ygWand.h"
#include "ygSound.h"
#include "ygSpace.h"
#include "ygEnvironment.h"
#include "ygCAVEHead.h"
#include "ygCAVENavigator.h"
#include "ygCAVETracker.h"
#include "ygCAVEWand.h"
#include "ygDSOLoader.h"
#include "ygNet.h"
#include "ygNodeDB.h"

using namespace std;


struct _ygNodeDBPrivateData
	{
	map<ygString,ygNode*> nodes;
	map<ygString, vector<ygNode*> > nodesByType;
	map<ygString,ygNodeConstructorPtr> nodeConstructor;
	};

struct _ygNodeDBPrivateData* ygNodeDB::p_ = NULL;



void ygNodeDB::init(void)
	{
	ygNet::init();
	if (!p_)
		p_ = new struct _ygNodeDBPrivateData;
	p_->nodeConstructor["Group"] = construct_ygNode;
	p_->nodeConstructor["ygGroup"] = construct_ygNode;
	p_->nodeConstructor["Node"] = construct_ygNode;
	p_->nodeConstructor["ygNode"] = construct_ygNode;
	p_->nodeConstructor["Transform"] = construct_ygTransform;
	p_->nodeConstructor["ygTransform"] = construct_ygTransform;
	p_->nodeConstructor["SimpleTransform"] = construct_ygSimpleTransform;
	p_->nodeConstructor["ygSimpleTransform"] = construct_ygSimpleTransform;
	p_->nodeConstructor["Light"] = construct_ygLight;
	p_->nodeConstructor["ygLight"] = construct_ygLight;
	p_->nodeConstructor["Object"] = construct_ygObject;
	p_->nodeConstructor["ygObject"] = construct_ygObject;
	p_->nodeConstructor["Switch"] = construct_ygSwitch;
	p_->nodeConstructor["ygSwitch"] = construct_ygSwitch;
	p_->nodeConstructor["Selector"] = construct_ygSelector;
	p_->nodeConstructor["ygSelector"] = construct_ygSelector;
	p_->nodeConstructor["User"] = construct_ygUser;
	p_->nodeConstructor["ygUser"] = construct_ygUser;
	p_->nodeConstructor["CAVEHead"] = construct_ygCAVEHead;
	p_->nodeConstructor["ygCAVEHead"] = construct_ygCAVEHead;
	p_->nodeConstructor["Wand"] = construct_ygWand;
	p_->nodeConstructor["ygWand"] = construct_ygWand;
	p_->nodeConstructor["CAVENavigator"] = construct_ygCAVENavigator;
	p_->nodeConstructor["ygCAVENavigator"] = construct_ygCAVENavigator;
	p_->nodeConstructor["CAVETracker"] = construct_ygCAVETracker;
	p_->nodeConstructor["ygCAVETracker"] = construct_ygCAVETracker;
	p_->nodeConstructor["CAVEWand"] = construct_ygCAVEWand;
	p_->nodeConstructor["ygCAVEWand"] = construct_ygCAVEWand;
	p_->nodeConstructor["ygSound"] = construct_ygSound;
	p_->nodeConstructor["Sound"] = construct_ygSound;
	p_->nodeConstructor["ygSpace"] = construct_ygSpace;
	p_->nodeConstructor["Space"] = construct_ygSpace;
	p_->nodeConstructor["ygEnvironment"] = construct_ygEnvironment;
	p_->nodeConstructor["Environment"] = construct_ygEnvironment;
	}


ygNode* ygNodeDB::find(const ygString& name,bool connect)
	{
	ygNode* node = p_->nodes[name];
	if ((!node) && (connect))
		node = ygNet::connectNewNode(name);
	return node;
	}


ygNode * ygNodeDB::create(const ygString& nodeType,const ygString& nodeName,
			  ygTokenStream *tStream)
	{
	if (p_->nodes[nodeName] != NULL)
		cerr << "ERROR: ygNodeDB::create: a node named '" << nodeName
			<< "' already exists\n";
	ygNode *node = NULL;
	ygNodeConstructorPtr constructPtr = p_->nodeConstructor[nodeType];
	if (!constructPtr)
		{
		constructPtr = ygDSOLoader::getNodeConstructor(nodeType);
		p_->nodeConstructor[nodeType] = constructPtr;
		}
	if (constructPtr)
		node = (*constructPtr)(nodeName.c_str(),true);
	if (!node)
		{
// NOTE: This may cause problems on SGIs when using STL strings!!
		cerr << "ERROR: ygNodeDB::create: unknown node type '"
			<< nodeType << "'";
		if (tStream)
			cerr << " (file " << tStream->filename() << ", line "
				<< tStream->linenumber() << ")";
		cerr << " - replacing with Group" << endl;
		node = new ygNode(nodeName.c_str());
		}
	if (node)
		{
		p_->nodes[nodeName] = node;
		p_->nodesByType[nodeType].push_back(node);
		ygNet::storeNode(node);
		}
	return node;
	}


ygNode * ygNodeDB::createProxy(const ygString& proxyType,const ygString& nodeName)
	{
	ygNode *node = NULL;
	ygNodeConstructorPtr constructPtr = p_->nodeConstructor[proxyType];
	if (!constructPtr)
		{
		constructPtr = ygDSOLoader::getNodeConstructor(proxyType);
		p_->nodeConstructor[proxyType] = constructPtr;
		}
	if (constructPtr)
		node = (*constructPtr)(nodeName.c_str(),false);
	if (!node)
		{
		cerr << "ERROR: ygNodeDB::createProxy: unknown node type '"
			<< proxyType << "' - replacing with Group" << endl;
		node = new ygNode(nodeName.c_str(),false);
		}
	if (node)
		{
		p_->nodes[nodeName] = node;
		p_->nodesByType[proxyType].push_back(node);
		node->requestNetKeys();
		}
	return node;
	}


const vector<ygNode*>& ygNodeDB::nodesOfType(const ygString& nodeType)
	{
	return p_->nodesByType[nodeType];
	}
