#include <Performer/pf/pfGroup.h>
#include <Performer/pfdu.h>
#include "ygWorld.h"
#include "ygNetKeys.h"
#include "ygPFObjectCache.h"
#include "ygObject.h"

extern "C" ygNode* construct_ygObject(const char* name,bool master) { return new ygObject(name,master); }

struct _ygObjectPrivateData
	{
	ygString filename;
	pfNode *obj;
	uint isectMask;
	bool drawn, cache, showLocal;
	ygString netFilename;
	};


ygObject::ygObject(const char* name,bool master) : ygTransform(name,master)
	{
	setClassName("ygObject",true);
	p_ = new struct _ygObjectPrivateData;
	p_->obj = NULL;
	p_->isectMask = YG_ISECT_ALLMASK;
	p_->drawn = true;
	p_->cache = true;
	p_->showLocal = true;
	addNetKey("file",&p_->netFilename,YG_NET_STRING);
	addNetKey("isect",&p_->isectMask,YG_NET_INT);
	addNetKey("drawn",&p_->drawn,YG_NET_BOOL);
	}


ygObject::~ygObject(void)
	{
	delete p_;
	}


void ygObject::reset(void)
	{
	p_->isectMask = YG_ISECT_ALLMASK;
	netKeyChanged("isect");
	if (p_->obj)
		{
		p_->obj->setTravMask(PFTRAV_ISECT, p_->isectMask,
					PFTRAV_SELF|PFTRAV_DESCEND,
					PF_SET);
		if ((p_->drawn) && (p_->showLocal))
			p_->obj->setTravMask(PFTRAV_DRAW, 0xffffffff,
					PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		else
			p_->obj->setTravMask(PFTRAV_DRAW, 0,
					PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		}
	ygTransform::reset();
	}


void ygObject::message(const ygMessage& msg)
	{
	if (msg == "file")
		{
		if (msg.args.size() == 1)
			loadFile(msg.args[0]);
		else
			msg.error(name(),"(wrong number of arguments)");
		}
	else if (msg == "wall")
		{
		if (msg.args.size() > 0)
			setWall(msg.boolArg(0));
		else
			setWall();
		}
	else if (msg == "floor")
		{
		if (msg.args.size() > 0)
			setFloor(msg.boolArg(0));
		else
			setFloor();
		}
	else if (msg == "draw")
		{
		if (msg.args.size() > 0)
			setDraw(msg.boolArg(0));
		else
			setDraw();
		}
	else if (msg == "showLocal")
		{
		if (msg.args.size() > 0)
			showLocal(msg.boolArg(0));
		else
			showLocal();
		}
	else if (msg == "cache")
		{
		if (msg.args.size() > 0)
			p_->cache = msg.boolArg(0);
		else
			p_->cache = true;
		}
	else
		ygTransform::message(msg);
	}


void ygObject::loadFile(const ygString& file)
	{
	if (p_->filename == file)
		return;
	if (p_->obj)
		pfnode()->removeChild(p_->obj);
	if (p_->cache)
		p_->obj = ygPFObjectCache::objectReference(file.c_str());
	else
		p_->obj = pfdLoadFile(file.c_str());
	if (p_->obj)
		{
		p_->filename = file;
		pfnode()->addChild(p_->obj);
		if ((p_->drawn) && (p_->showLocal))
			p_->obj->setTravMask(PFTRAV_DRAW, 0xffffffff,
					PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		else
			p_->obj->setTravMask(PFTRAV_DRAW, 0,
					PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		p_->obj->setTravMask(PFTRAV_ISECT, p_->isectMask,
					PFTRAV_SELF|PFTRAV_DESCEND|PFTRAV_IS_CACHE, PF_SET);
		p_->netFilename = file;
		netKeyChanged("file");
		}
	}


void ygObject::setWall(bool val)
	{
	if (val)
		p_->isectMask |= YG_ISECT_WALLMASK;
	else
		p_->isectMask &= ~YG_ISECT_WALLMASK;
	netKeyChanged("isect");
	if (p_->obj)
		p_->obj->setTravMask(PFTRAV_ISECT, p_->isectMask,
				PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
	}


void ygObject::setFloor(bool val)
	{
	if (val)
		p_->isectMask |= YG_ISECT_FLOORMASK;
	else
		p_->isectMask &= ~YG_ISECT_FLOORMASK;
	netKeyChanged("isect");
	if (p_->obj)
		p_->obj->setTravMask(PFTRAV_ISECT, p_->isectMask,
				PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
	}


void ygObject::setDraw(bool val)
	{
	p_->drawn = val;
	netKeyChanged("drawn");
	if (p_->obj)
		{
		if (p_->drawn)
			p_->obj->setTravMask(PFTRAV_DRAW, 0xffffffff,
					PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		else
			p_->obj->setTravMask(PFTRAV_DRAW, 0,
					PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		}
	}


void ygObject::showLocal(bool show)
	{
	p_->showLocal = show;
	if (p_->obj)
		{
		if (show)
			p_->obj->setTravMask(PFTRAV_DRAW, 0xffffffff, PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		else
			p_->obj->setTravMask(PFTRAV_DRAW, 0, PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		}
	}

const ygString& ygObject::filename(void) const
	{
	return p_->filename;
	}


void ygObject::acceptNetKey(const ygString& key)
	{
	if (key == "file")
		loadFile(p_->netFilename);
	else if (key == "drawn")
		setDraw(p_->drawn);
	else if (key == "isect")
		{
		if (p_->obj)
			p_->obj->setTravMask(PFTRAV_ISECT, p_->isectMask,
					PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		}
	else
		ygTransform::acceptNetKey(key);
	}
