#include <Performer/pf/pfDCS.h>
#include "ygNetKeys.h"
#include "ygWorld.h"
#include "ygSimpleTransform.h"

extern "C" ygNode* construct_ygSimpleTransform(const char* name,bool master) { return new ygSimpleTransform(name,master); }

#define COORD_NETKEY "x"


struct _ygSimpleTransformPrivateData
	{
	pfDCS * dcs;
	pfCoord netCoord;
	float updateInterval, lastUpdate;
	};


ygSimpleTransform::ygSimpleTransform(const char* name,bool master) : ygNode(name,master,false)
	{
	setClassName("ygSimpleTransform",true);
	p_ = new struct _ygSimpleTransformPrivateData;
	p_->dcs = new pfDCS;
	setPfNode(p_->dcs);
	p_->netCoord.xyz.set(0,0,0);
	p_->netCoord.hpr.set(0,0,0);
	p_->updateInterval = 0;
	p_->lastUpdate = 0;
	addNetKey(COORD_NETKEY, &p_->netCoord, YG_NET_COORD);
	unreliableKey(COORD_NETKEY);
	}


ygSimpleTransform::~ygSimpleTransform(void)
	{
	delete p_;
	}


void ygSimpleTransform::reset(void)
	{
	if (isNetMaster())
		{
		pfMatrix mat;
		mat.makeIdent();
		setMatrix(mat);
		}
	ygNode::reset();
	}


/*****  DCS functions *****/

void ygSimpleTransform::getMatrix(pfMatrix& mat) const
	{
	p_->dcs->getMat(mat);
	}


void ygSimpleTransform::setMatrix(pfMatrix& mat)
	{
	p_->dcs->setMat(mat);
	}


void ygSimpleTransform::setPosition(float x,float y,float z)
	{
	p_->dcs->setTrans(x,y,z);
	}


void ygSimpleTransform::setOrientation(float x,float y,float z)
	{
	p_->dcs->setRot(z,x,y);
	}


/***** Message functions *****/

void ygSimpleTransform::message(const ygMessage& msg)
	{
	if (msg == "position")
		setPosition(msg);
	else if (msg == "orientation")
		setOrientation(msg);
	else if (msg == "updateInterval")
		{
		if (msg.args.size() > 0)
			p_->updateInterval = msg.floatArg(0);
		}
	else
		ygNode::message(msg);
	}


void ygSimpleTransform::setPosition(const ygMessage& msg)
	{
	pfVec3 v;
	if (msg.getVec3Args(v))
		setPosition(v);
	else
		msg.error(name(),"(wrong number of arguments)");
	}


void ygSimpleTransform::setOrientation(const ygMessage& msg)
	{
	pfVec3 v;
	if (msg.getVec3Args(v))
		setOrientation(v);
	else
		msg.error(name(),"(wrong number of arguments)");
	}


/***** Net functions *****/

void ygSimpleTransform::acceptNetKey(const ygString& key)
	{
	if (key == COORD_NETKEY)
		{
		setPosition(p_->netCoord.xyz);
		setOrientation(p_->netCoord.hpr[1],p_->netCoord.hpr[2],p_->netCoord.hpr[0]);
		}
	else
		ygNode::acceptNetKey(key);
	}


void ygSimpleTransform::app(void)
	{
	if (isNetMaster())
		{
		if ((ygWorld::FrameTime - p_->lastUpdate) > p_->updateInterval)
			{
			pfMatrix matrix;
			pfCoord coord;
			getMatrix(matrix);
			matrix.getOrthoCoord(&coord);
			if ((coord.xyz != p_->netCoord.xyz) ||
			    (coord.hpr != p_->netCoord.hpr))
				{
				p_->netCoord = coord;
				netKeyChanged(COORD_NETKEY);
				p_->lastUpdate = ygWorld::FrameTime;
				}
			}
		}
	ygNode::app();
	}
