#include <Performer/pf/pfGroup.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pr/pfGeoSet.h>
#include <Performer/pfdu.h>
#include "ygUtil.h"
#include "ygNetKeys.h"
#include "ygInfiniteVolume.h"
#include "ygSpace.h"

using namespace std;

extern "C" ygNode* construct_ygSpace(const char* name,bool master) { return new ygSpace(name,master); }

struct _ygSpacePrivateData
	{
	bool debugVolume;
	pfGeode * debugGeode;
	ygString netString;
	};


ygSpace::ygSpace(const char* name,bool master) : ygNode(name,master)
	{
	setClassName("ygSpace",true);
	p_ = new struct _ygSpacePrivateData;
	volume_ = new ygInfiniteVolume;
	p_->debugGeode = NULL;
	debugFlag("volume", &p_->debugVolume);
	ygWriteVolumeMessage(volume_,p_->netString);
	addNetKey("volume", &p_->netString, YG_NET_STRING);
	}

ygSpace::~ygSpace(void)
	{
	if (volume_)
		delete volume_;
	}


void ygSpace::reset(void)
	{
	setVolume(new ygInfiniteVolume);
	ygNode::reset();
	}


void ygSpace::message(const ygMessage& msg)
	{
	if (msg == "volume")
		setVolume(ygParseVolumeMessage(msg));
	else
		ygNode::message(msg);
	}


void ygSpace::setVolume(ygVolume * vol)
	{
	if (volume_)
		delete volume_;
	if (vol)
		volume_ = vol;
	else
		volume_ = new ygInfiniteVolume;
	if (p_->debugVolume)
		makeDebugOutline();
	ygWriteVolumeMessage(volume_,p_->netString);
	netKeyChanged("volume");
	}


void ygSpace::makeDebugOutline(void)
	{
	pfGeoSet * gset;
	if (p_->debugGeode)
		{
		gset = p_->debugGeode->getGSet(0);
		if (gset)
			{
			p_->debugGeode->removeGSet(gset);
			pfDelete(gset);
			}
		}
	else
		{
		p_->debugGeode = new pfGeode;
		pfnode()->addChild(p_->debugGeode);
		if (!p_->debugGeode)
			{
			cerr << "ERROR: ygSpace::makeDebugOutline: failed to"
				" create debugGeode\n";
			return;
			}
		}
	gset = ygCreateVolumeGeoset(volume_,YG_STYLE_WIREFRAME);
	if (gset)
		{
		p_->debugGeode->addGSet(gset);
		p_->debugGeode->setTravMask(PFTRAV_ISECT, 0, 
					PFTRAV_SELF|PFTRAV_DESCEND, PF_SET);
		}
	}


void ygSpace::acceptNetKey(const ygString& key)
	{
	if (key == "volume")
		{
		ygMessage msg;
		msg.parseString(p_->netString);
		setVolume(ygParseVolumeMessage(msg));
		}
	else
		ygNode::acceptNetKey(key);
	}
