#include <Performer/pf/pfSwitch.h>
#include "ygNetKeys.h"
#include "ygSwitch.h"

extern "C" ygNode* construct_ygSwitch(const char* name,bool master) { return new ygSwitch(name,master); }

struct _ygSwitchPrivateData
	{
	pfSwitch * sw;
	bool prevOn;
	bool netOn;
	};


ygSwitch::ygSwitch(const char* name,bool master) : ygNode(name,master,false)
	{
	setClassName("ygSwitch",true);
	p_ = new struct _ygSwitchPrivateData;
	p_->sw = new pfSwitch;
	setPfNode(p_->sw);
	addNetKey("on", &p_->netOn, YG_NET_BOOL);
	reset();
	}


ygSwitch::~ygSwitch(void)
	{
	delete p_;
	}


void ygSwitch::reset(void)
	{
	if (isNetMaster())
		{
		on();
		p_->prevOn = true;
		}
	ygNode::reset();
	}


void ygSwitch::app(void)
	{
	bool ison = isOn();
	if ((ison) && (!p_->prevOn))
		eventOccurred("SwitchOn");
	else if ((!ison) && (p_->prevOn))
		eventOccurred("SwitchOff");
	p_->prevOn = ison;
	ygNode::app();
	}


void ygSwitch::appTraverse(void)
	{
	timingBegin();
	app();
	if (isOn())
		{
		for (int i=0; i < numChildren(); i++)
			child(i)->appTraverse();
		}
	timingEnd();
	}


/*****  Switch functions *****/

bool ygSwitch::isOn(void) const
	{
	return (p_->sw->getVal() == PFSWITCH_ON);
	}


void ygSwitch::on(void)
	{
	if (!isOn())
		{
		p_->sw->setVal(PFSWITCH_ON);
		p_->netOn = true;
		netKeyChanged("on");
		}
	}


void ygSwitch::off(void)
	{
	if (isOn())
		{
		p_->sw->setVal(PFSWITCH_OFF);
		p_->netOn = false;
		netKeyChanged("on");
		}
	}


void ygSwitch::toggle(void)
	{
	if (isOn())
		off();
	else
		on();
	}


/***** Message functions *****/

void ygSwitch::message(const ygMessage& msg)
	{
	if (msg == "on")
		on();
	else if (msg == "off")
		off();
	else if (msg == "toggle")
		toggle();
	else
		ygNode::message(msg);
	}


/***** Net functions *****/

void ygSwitch::acceptNetKey(const ygString& key)
	{
	if (key == "on")
		{
		if (p_->netOn)
			on();
		else
			off();
		}
	else
		ygNode::acceptNetKey(key);
	}
