#include <Performer/pf/pfDCS.h>
#include "ygNetKeys.h"
#include "ygTransform.h"

extern "C" ygNode* construct_ygTransform(const char* name,bool master) { return new ygTransform(name,master); }

#define MATRIX_NETKEY "m"


struct _ygTransformPrivateData
	{
	pfDCS * dcs;
	pfMatrix netMatrix;
	};


ygTransform::ygTransform(const char* name,bool master) : ygNode(name,master,false)
	{
	setClassName("ygTransform",true);
	p_ = new struct _ygTransformPrivateData;
	p_->dcs = new pfDCS;
	setPfNode(p_->dcs);
	p_->netMatrix.makeIdent();
	addNetKey(MATRIX_NETKEY, &p_->netMatrix, YG_NET_MATRIX);
	}


ygTransform::~ygTransform(void)
	{
	delete p_;
	}


void ygTransform::reset(void)
	{
	if (isNetMaster())
		{
		pfMatrix mat;
		mat.makeIdent();
		setMatrix(mat);
		}
	ygNode::reset();
	}


/*****  DCS functions *****/

void ygTransform::getMatrix(pfMatrix& mat) const
	{
	p_->dcs->getMat(mat);
	}


void ygTransform::setMatrix(pfMatrix& mat)
	{
	p_->dcs->setMat(mat);
	storeMatrixKey();
	}


void ygTransform::setPosition(float x,float y,float z)
	{
	p_->dcs->setTrans(x,y,z);
	storeMatrixKey();
	}


void ygTransform::setOrientation(float x,float y,float z)
	{
	p_->dcs->setRot(z,x,y);
	storeMatrixKey();
	}


void ygTransform::setSize(float s)
	{
	p_->dcs->setScale(s);
	storeMatrixKey();
	}


void ygTransform::setSize(float x,float y,float z)
	{
	p_->dcs->setScale(x,y,z);
	storeMatrixKey();
	}


/***** Message functions *****/

void ygTransform::message(const ygMessage& msg)
	{
	if (msg == "position")
		setPosition(msg);
	else if (msg == "orientation")
		setOrientation(msg);
	else if (msg == "size")
		setSize(msg);
	else
		ygNode::message(msg);
	}


void ygTransform::setPosition(const ygMessage& msg)
	{
	pfVec3 v;
	if (msg.getVec3Args(v))
		setPosition(v);
	else
		msg.error(name(),"(wrong number of arguments)");
	}


void ygTransform::setOrientation(const ygMessage& msg)
	{
	pfVec3 v;
	if (msg.getVec3Args(v))
		setOrientation(v);
	else
		msg.error(name(),"(wrong number of arguments)");
	}


void ygTransform::setSize(const ygMessage& msg)
	{
	pfVec3 v;
	if (msg.args.size() == 1)
		setSize(msg.floatArg(0));
	else if (msg.getVec3Args(v))
		setSize(v);
	else
		msg.error(name(),"(wrong number of arguments)");
	}


/***** Net functions *****/

void ygTransform::acceptNetKey(const ygString& key)
	{
	if (key == MATRIX_NETKEY)
		setMatrix(p_->netMatrix);
	else
		ygNode::acceptNetKey(key);
	}


void ygTransform::storeMatrixKey(void)
	{
	if (isNetMaster())
		{
		getMatrix(p_->netMatrix);
		netKeyChanged(MATRIX_NETKEY);
		}
	}
