#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <sys/time.h>
#include "ygNode.h"
#include "ygFileServer.h"
#include "ygDSOLoader.h"
#include "ygUtil.h"

char * ygVersion = "Ygdrasil Version 0.1.5 - compiled " __DATE__;

using namespace std;


bool ygFindFileInYgPath(const ygString& filename, ygString& returnPath)
	{
	static ygString searchPath;
	if (searchPath.length() < 1)
		{
		if (getenv("YG_PATH"))
			searchPath = getenv("YG_PATH");
		else
			searchPath = ".";
		}
	return ygFindFile(filename,searchPath,returnPath);
	}


bool ygFindFile(const ygString& filename, const ygString& searchPath,
	               ygString& returnPath)
	{
	ygString testName(filename);
	ygString *dir;
	int i=0;
	struct stat statbuf;
	if (stat(testName.c_str(),&statbuf) == 0)
		{
		returnPath = testName;
		return true;
		}
	while (dir = searchPath.nextToken(":",&i))
		{
		testName = *dir;
		delete dir;
		testName += "/";
		testName += filename;
		if (stat(testName.c_str(),&statbuf) == 0)
			{
			returnPath = testName;
			return true;
			}
		}
	return false;
	}


static bool ygTimeInitted=false;
static struct timeval ygTimeStart;
static float ygTimeMicrosecondF=1.0f/1000000.0f;
static double ygTimeMicrosecondD=1.0/1000000.0;

float ygGetTime(void)
	{
	struct timeval t;
	if (!ygTimeInitted)
		{
		gettimeofday(&ygTimeStart,NULL);
		ygTimeInitted = true;
		}
	gettimeofday(&t,NULL);
	return (t.tv_sec - ygTimeStart.tv_sec) +
		(t.tv_usec - ygTimeStart.tv_usec) * ygTimeMicrosecondF;
	}

double ygGetTimeD(void)
	{
	struct timeval t;
	if (!ygTimeInitted)
		{
		gettimeofday(&ygTimeStart,NULL);
		ygTimeInitted = true;
		}
	gettimeofday(&t,NULL);
	return (t.tv_sec - ygTimeStart.tv_sec) +
		(t.tv_usec - ygTimeStart.tv_usec) * ygTimeMicrosecondD;
	}


static void printTree(ygNode * node, int depth)
	{
	if (node)
		{
		int i;
		for (i=0; i < depth; i++)
			cout << "  ";
		cout << node->className() << ":" << node->name() << endl;
		for (i=0; i < node->numChildren(); i++)
			printTree(node->child(i),depth+1);
		}
	}


void ygPrintSceneGraph(ygNode * node)
	{
	cout << "Scene Graph:\n";
	printTree(node, 1);
	}


static bool ygServerMode_ = false;

bool ygServerMode(void)
	{
	return ygServerMode_;
	}


void ygSetServerMode(bool val)
	{
	ygServerMode_ = val;
	}


void ygPrePFInit(void)
	{
#ifdef __sgi
	extern void ygSGIIOStreamInit(void);
	ygSGIIOStreamInit();
#endif
	cout << ygVersion << endl;
	}


void ygPrePFConfig(void)
	{
	ygFileServer::initialize();
	ygDSOLoader::init();
	}


#ifdef __sgi

bool ygCreateThread(ygThreadFuncPtr func,void *arg)
	{
	pid_t p = sproc((void(*)(void*))func,PR_SADDR,arg);
	if (p <= 0)
		{
		perror("ygCreateThread: sproc");
		return false;
		}
	return true;
	}

#else

#include <CAVERN.hxx>

bool ygCreateThread(ygThreadFuncPtr func,void *arg)
	{
	CAVERNts_thread_c *thread = new CAVERNts_thread_c;
	int retval = thread->create(func,arg);
	return (retval == 0);
	}

#endif
