#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <Xm/MainW.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/DialogS.h>
#include <Xm/FileSB.h>
#include <Xm/Scale.h>
#include <Xm/TextF.h>
#include <Xm/RowColumn.h>
#include "ygUDPSocket.h"

struct xformData
        {
        float x, y, z;
        };
#define X_SLIDER 1
#define Y_SLIDER 2
#define Z_SLIDER 3

#define X_MAX 100
#define Y_MAX 100
#define Z_MAX 100


static struct xformData *controls;
ygUDPSocket *udpsocket;

struct _guiData
	{
	Widget main_w;
	Widget x_textf;
	Widget y_textf;
	Widget z_textf;
	} guiData;

#define MOTIF_TYPED_LABEL(s) XtVaTypedArg,XmNlabelString,XmRString,s,strlen(s)+1
#define MOTIF_TYPED_TITLE(s) XtVaTypedArg,XmNtitleString,XmRString,s,strlen(s)+1


static void create_work_area(Widget main_w);
static void create_file_menu(Widget menubar,int buttonnum);
static void create_edit_menu(Widget menubar,int buttonnum);
static void create_help_menu(Widget menubar,int buttonnum);
static void create_menus(Widget mainw);
static Widget create_a_slider(Widget parent_w,char *label,int min,int max,int,void *data);
static void slider_change_cb(Widget w,void *data,XmScaleCallbackStruct *cbs);
static void slider_drag_cb(Widget w,void *data,XmScaleCallbackStruct *cbs);
static void redraw_slider_text(Widget w,float val);
static void cleanup_and_quit(void);

void usage(char *av0)
{
 fprintf(stderr,"Usage: %s \n",av0);
 exit(-1);
}

main(int argc,char **argv)
{
 XtAppContext app;
 Widget toplevel;
/* String fallbacks[] = { "*scale*fontList: -*-helvetica-medium-r-*--10-*",
			"*layerlabel*fontList: -*-times-bold-i-*--12-*",
			NULL }; */
#define fallbacks NULL
 toplevel = XtVaAppInitialize(&app,argv[0],NULL,0,&argc,argv,fallbacks,NULL);
 udpsocket  = new ygUDPSocket(0);
 udpsocket->setDestination("localhost",1967);
 controls = new struct xformData;
 controls->x = controls->y = 0;
 controls->z = 0;
 guiData.main_w = XtVaCreateManagedWidget("main",xmMainWindowWidgetClass,toplevel,
		NULL);
 create_menus(guiData.main_w);
 create_work_area(guiData.main_w);
 XtRealizeWidget(toplevel);
 XtAppMainLoop(app);
}


static void cleanup_and_quit(void)
{
 exit(0);
}

static void create_work_area(Widget main_w)
{
 Widget box_w;
 box_w = XtVaCreateManagedWidget("rowcol",xmRowColumnWidgetClass,main_w,
		XmNwidth, 600,
		XmNheight, 250,
		NULL);
 guiData.x_textf = create_a_slider(box_w,"R",0,X_MAX,controls->x,
				(void*)X_SLIDER);
 guiData.y_textf = create_a_slider(box_w,"G",0,Y_MAX,controls->y,
				(void*)Y_SLIDER);
 guiData.z_textf = create_a_slider(box_w,"B",0,Z_MAX,controls->z,
				(void*)Z_SLIDER);
 XtVaSetValues(main_w,XmNworkWindow,box_w,NULL);
}

static Widget create_a_slider(Widget parent_w,char *label,int min,int max,int initval,void *data)
{
 Widget scale_w,form_w,textf_w;
 char str[32];
 form_w = XtVaCreateManagedWidget("form",xmFormWidgetClass,parent_w,
		NULL);
 scale_w = XtVaCreateManagedWidget("scale",xmScaleWidgetClass,form_w,
		MOTIF_TYPED_TITLE(label),
		XmNminimum, min,
		XmNmaximum, max,
		XmNvalue, initval,
		XmNshowValue, False,
		XmNorientation, XmHORIZONTAL,
		XmNscaleWidth, 300,
		XmNleftAttachment, XmATTACH_FORM,
		XmNtopAttachment, XmATTACH_FORM,
		XmNtopOffset, 2,
		NULL);
 XtAddCallback(scale_w,XmNvalueChangedCallback,(XtCallbackProc)slider_change_cb,
			data);
 XtAddCallback(scale_w,XmNdragCallback,(XtCallbackProc)slider_drag_cb,
			data);
 sprintf(str,"%d",initval);
 textf_w = XtVaCreateManagedWidget("textf",xmTextFieldWidgetClass,form_w,
		XmNvalue, str,
		XmNcolumns, 8,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, scale_w,
		XmNtopAttachment, XmATTACH_FORM,
		XmNtopOffset, 2,
		NULL);
 return textf_w;
}

static void slider_change_cb(Widget w,void *data,XmScaleCallbackStruct *cbs)
{
 int slider = (int)data;
 if (slider == X_SLIDER)
	{
	controls->x = cbs->value / 100.0f;
	redraw_slider_text(guiData.x_textf,controls->x);
	}
 else if (slider == Y_SLIDER)
	{
	controls->y = cbs->value / 100.0f;
	redraw_slider_text(guiData.y_textf,controls->y);
	}
 else if (slider == Z_SLIDER)
	{
	controls->z = cbs->value / 100.0f;
	redraw_slider_text(guiData.z_textf,controls->z);
	}
 char message[256];
 sprintf(message,"sun.diffuse(%.2f %.2f %.2f)",controls->x,controls->y,controls->z);
 udpsocket->send((unsigned char *)message,strlen(message)+1);
}

static void slider_drag_cb(Widget w,void *data,XmScaleCallbackStruct *cbs)
{
 int slider = (int)data;
 if (slider == X_SLIDER)
	{
	controls->x = cbs->value / 100.0f;
	redraw_slider_text(guiData.x_textf,controls->x);
	}
 else if (slider == Y_SLIDER)
	{
	controls->y = cbs->value / 100.0f;
	redraw_slider_text(guiData.y_textf,controls->y);
	}
 else if (slider == Z_SLIDER)
	{
	controls->z = cbs->value / 100.0f;
	redraw_slider_text(guiData.z_textf,controls->z);
	}
 char message[256];
 sprintf(message,"sun.diffuse(%.2f %.2f %.2f)",controls->x,controls->y,controls->z);
 udpsocket->send((unsigned char *)message,strlen(message)+1);
}

static void redraw_slider_text(Widget w,float val)
{
 char str[32];
 sprintf(str,"%.2f",val);
 XmTextFieldSetString(w,str);
}

/* file_menu_cb
	Purpose: Callback function called when an item is selected from the
		"File" menu. The function corresponding to the selected item
		is called.
	Example call: file_menu_cb(menu_item,itemnum);
	Arguments: Widget menu_item: the menu item's widget
		   int itemnum: the index of the item selected
	Called by: callback for menu created in create_file_menu
	Author: Dave Pape
*/
static void file_menu_cb(Widget menu_item,int itemnum)
{
 switch (itemnum) {
        case 0: 
        case 1: 
        case 2: cleanup_and_quit();
        }
}

/* create_file_menu
	Purpose: Creates the "File" menu on the main window's menu bar. This
		menu contains entries for loading and saving specifications,
		and for quitting.
	Example call: create_file_menu(menubar,buttonnum);
	Arguments: Widget menubar: the parent menu bar of this menu
		   int buttonnum: the index of this menu on the menu bar
	Called by: create_menus
	Author: Dave Pape
*/
static void create_file_menu(Widget menubar,int buttonnum)
{
 XmString quit;
 quit = XmStringCreateSimple("Quit");
 XmVaCreateSimplePulldownMenu(menubar,"file_menu",buttonnum,
		(XtCallbackProc)file_menu_cb,
                XmVaPUSHBUTTON, quit, 'Q', NULL, NULL,
                NULL);
 XmStringFree(quit);
}

static void create_edit_menu(Widget menubar,int buttonnum)
{
}

static void create_help_menu(Widget menubar,int buttonnum)
{
}


/* create_menus
	Purpose: Creates the main window's menu bar & menus.
	Example call: create_menus(main_w);
	Arguments: Widget main_w: the main window widget
	Called by: create_gui
	Author: Dave Pape
*/
static void create_menus(Widget main_w)
{
 XmString file,edit,help;
 Widget menubar,w;
 file = XmStringCreateSimple("File");
 edit = XmStringCreateSimple("Edit");
 help = XmStringCreateSimple("Help");
 menubar = XmVaCreateSimpleMenuBar(main_w,"menubar",
                XmVaCASCADEBUTTON, file, 'F',
                XmVaCASCADEBUTTON, edit, 'E',
                XmVaCASCADEBUTTON, help, 'H',
                NULL);
 XmStringFree(file);
 XmStringFree(edit);
 XmStringFree(help);
 if (w=XtNameToWidget(menubar,"button_3"))
	XtVaSetValues(menubar,XmNmenuHelpWidget,w,NULL);
 create_file_menu(menubar,0);
 create_edit_menu(menubar,1);
 create_help_menu(menubar,2);
 XtManageChild(menubar);
 XtVaSetValues(main_w,XmNmenuBar,menubar,NULL);
}
